##Copyright (c) 2002, Fedor Baart & Hans de Wit (Stichting Farmaceutische Kengetallen)
##All rights reserved.
##
##Redistribution and use in source and binary forms, with or without modification,
##are permitted provided that the following conditions are met:
##
##Redistributions of source code must retain the above copyright notice, this
##list of conditions and the following disclaimer.
##
##Redistributions in binary form must reproduce the above copyright notice,
##this list of conditions and the following disclaimer in the documentation and/or
##other materials provided with the distribution.
##
##Neither the name of the Stichting Farmaceutische Kengetallen nor the names of
##its contributors may be used to endorse or promote products derived from this
##software without specific prior written permission.
##
##THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
##AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
##IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
##DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
##FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
##DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
##SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
##CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
##OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
##OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
##
##Thanks to Gerald Rosennfellner for his help and useful comments.
'''
SVGdraw.py - generate SVG drawings
======================================================

:Author: Fedor Baart & Hans de Wit
:Release: $Id$
:Date: |today|
:Tags: Python

This module has been copied from 3rd party resources.

SVGdraw uses an object model drawing and a method toXML to create SVG graphics
by using easy to use classes and methods usualy you start by creating a drawing eg

    d=drawing()
    #then you create a SVG root element
    s=svg()
    #then you add some elements eg a circle and add it to the svg root element
    c=circle()
    #you can supply attributes by using named arguments.
    c=circle(fill='red',stroke='blue')
    #or by updating the attributes attribute:
    c.attributes['stroke-width']=1
    s.addElement(c)
    #then you add the svg root element to the drawing
    d.setSVG(s)
    #and finaly you xmlify the drawing
    d.toXml()
    

this results in the svg source of the drawing, which consists of a circle
on a white background. Its as easy as that;)
This module was created using the SVG specification of www.w3c.org and the
O'Reilly (www.oreilly.com) python books as information sources. A svg viewer
is available from www.adobe.com
'''

__version__="1.0"

# there are two possibilities to generate svg:
# via a dom implementation and directly using <element>text</element> strings
# the latter is way faster (and shorter in coding)
# the former is only used in debugging svg programs
# maybe it will be removed alltogether after a while
# with the following variable you indicate whether to use the dom implementation
# Note that PyXML is required for using the dom implementation.
# It is also possible to use the standard minidom. But I didn't try that one.
# Anyway the text based approach is about 60 times faster than using the full dom implementation.
use_dom_implementation=0


import exceptions
if use_dom_implementation<>0:
    try:
        from xml.dom import implementation
        from xml.dom.ext import PrettyPrint
    except:
        raise exceptions.ImportError, "PyXML is required for using the dom implementation"
#The implementation is used for the creating the XML document.
#The prettyprint module is used for converting the xml document object to a xml file

import sys
assert sys.version_info[0]>=2
if sys.version_info[1]<2:
    True=1
    False=0
    file=open
    
sys.setrecursionlimit=50
#The recursion limit is set conservative so mistakes like s=svg() s.addElement(s)
#won't eat up too much processor time.

#the following code is pasted form xml.sax.saxutils
#it makes it possible to run the code without the xml sax package installed
#To make it possible to have <rubbish> in your text elements, it is necessary to escape the texts
def _escape(data, entities={}):
    """Escape &, <, and > in a string of data.

    You can escape other strings of data by passing a dictionary as
    the optional entities parameter.  The keys and values must all be
    strings; each key will be replaced with its corresponding value.
    """
    data = data.replace("&", "&amp;")
    data = data.replace("<", "&lt;")
    data = data.replace(">", "&gt;")
    for chars, entity in entities.items():
        data = data.replace(chars, entity)
    return data

def _quoteattr(data, entities={}):
    """Escape and quote an attribute value.

    Escape &, <, and > in a string of data, then quote it for use as
    an attribute value.  The \" character will be escaped as well, if
    necessary.

    You can escape other strings of data by passing a dictionary as
    the optional entities parameter.  The keys and values must all be
    strings; each key will be replaced with its corresponding value.
    """
    data = _escape(data, entities)
    if '"' in data:
        if "'" in data:
            data = '"%s"' % data.replace('"', "&quot;")
        else:
            data = "'%s'" % data
    else:
        data = '"%s"' % data
    return data



def _xypointlist(a):
    """formats a list of xy pairs"""
    s=''
    for e in a: #this could be done more elegant
        s+=str(e)[1:-1] +'  '
    return s

def _viewboxlist(a):
    """formats a tuple"""
    s=''
    for e in a: 
        s+=str(e)+' '
    return s

def _pointlist(a):
    """formats a list of numbers"""
    return str(a)[1:-1]

class pathdata:
    """class used to create a pathdata object which can be used for a path.
    although most methods are pretty straightforward it might be useful to look at the SVG specification."""
    #I didn't test the methods below. 
    def __init__(self,x=None,y=None):
        self.path=[]
        if x is not None and y is not None:
            self.path.append('M '+str(x)+' '+str(y))
    def closepath(self):
        """ends the path"""
        self.path.append('z')
    def move(self,x,y):
        """move to absolute"""
        self.path.append('M '+str(x)+' '+str(y))
    def relmove(self,x,y):
        """move to relative"""
        self.path.append('m '+str(x)+' '+str(y))
    def line(self,x,y):
        """line to absolute"""
        self.path.append('L '+str(x)+' '+str(y))
    def relline(self,x,y):
        """line to relative"""
        self.path.append('l '+str(x)+' '+str(y))
    def hline(self,x):
        """horizontal line to absolute"""
        self.path.append('H'+str(x))
    def relhline(self,x):
        """horizontal line to relative"""
        self.path.append('h'+str(x))
    def vline(self,y):
        """verical line to absolute"""
        self.path.append('V'+str(y))
    def relvline(self,y):
        """vertical line to relative"""
        self.path.append('v'+str(y))
    def bezier(self,x1,y1,x2,y2,x,y):
        """bezier with xy1 and xy2 to xy absolut"""
        self.path.append('C'+str(x1)+','+str(y1)+' '+str(x2)+','+str(y2)+' '+str(x)+','+str(y))
    def relbezier(self,x1,y1,x2,y2,x,y):
        """bezier with xy1 and xy2 to xy relative"""
        self.path.append('c'+str(x1)+','+str(y1)+' '+str(x2)+','+str(y2)+' '+str(x)+','+str(y))
    def smbezier(self,x2,y2,x,y):
        """smooth bezier with xy2 to xy absolut"""
        self.path.append('S'+str(x2)+','+str(y2)+' '+str(x)+','+str(y))
    def relsmbezier(self,x2,y2,x,y):
        """smooth bezier with xy2 to xy relative"""
        self.path.append('s'+str(x2)+','+str(y2)+' '+str(x)+','+str(y))
    def qbezier(self,x1,y1,x,y):
        """quadratic bezier with xy1 to xy absolut"""
        self.path.append('Q'+str(x1)+','+str(y1)+' '+str(x)+','+str(y))
    def relqbezier(self,x1,y1,x,y):
        """quadratic bezier with xy1 to xy relative"""
        self.path.append('q'+str(x1)+','+str(y1)+' '+str(x)+','+str(y))
    def smqbezier(self,x,y):
        """smooth quadratic bezier to xy absolut"""
        self.path.append('T'+str(x)+','+str(y))
    def relsmqbezier(self,x,y):
        """smooth quadratic bezier to xy relative"""
        self.path.append('t'+str(x)+','+str(y))
    def ellarc(self,rx,ry,xrot,laf,sf,x,y):
        """elliptival arc with rx and ry rotating with xrot using large-arc-flag and sweep-flag  to xy absolut"""
        self.path.append('A'+str(rx)+','+str(ry)+' '+str(xrot)+' '+str(laf)+' '+str(sf)+' '+str(x)+' '+str(y))
    def relellarc(self,rx,ry,xrot,laf,sf,x,y):
        """elliptival arc with rx and ry rotating with xrot using large-arc-flag and sweep-flag  to xy relative"""
        self.path.append('a'+str(rx)+','+str(ry)+' '+str(xrot)+' '+str(laf)+' '+str(sf)+' '+str(x)+' '+str(y))
    def __repr__(self):
        return ' '.join(self.path)
    


      
class SVGelement:
    """SVGelement(type,attributes,elements,text,namespace,**args)
    Creates a arbitrary svg element and is intended to be subclassed not used on its own.
    This element is the base of every svg element it defines a class which resembles
    a xml-element. The main advantage of this kind of implementation is that you don't
    have to create a toXML method for every different graph object. Every element
    consists of a type, attribute, optional subelements, optional text and an optional
    namespace. Note the elements==None, if elements = None:self.elements=[] construction.
    This is done because if you default to elements=[] every object has a reference
    to the same empty list."""
    def __init__(self,type='',attributes=None,elements=None,text='',namespace='',cdata=None,**args):
        self.type=type
        if attributes==None:
            self.attributes={}
        else:
            self.attributes=attributes
        if elements==None:
            self.elements=[]
        else:
            self.elements=elements
        self.text=text
        self.namespace=namespace
        self.cdata=cdata
        for arg in args.keys():
            self.attributes[arg]=args[arg]
    def addElement(self,SVGelement):
        """adds an element to a SVGelement

        SVGelement.addElement(SVGelement)
        """
        self.elements.append(SVGelement)

    def toXml(self,level,f):
        f.write('\t'*level)
        f.write('<'+self.type)
        for attkey in self.attributes.keys():
            f.write(' '+_escape(str(attkey))+'='+_quoteattr(str(self.attributes[attkey])))
        if self.namespace:
            f.write(' xmlns="'+ _escape(str(self.namespace))+'" ')
            ## added by Andreas Heger
            f.write(' xmlns:xlink="' + _escape(str("http://www.w3.org/1999/xlink")) + '" ')
        if self.elements or self.text or self.cdata:
            f.write('>')
        if self.elements:
            f.write('\n')
        for element in self.elements:
            element.toXml(level+1,f)
        if self.cdata:
            f.write('\n'+'\t'*(level+1)+'<![CDATA[')
            for line in self.cdata.splitlines():
               f.write('\n'+'\t'*(level+2)+line)
            f.write('\n'+'\t'*(level+1)+']]>\n')
        if self.text:
            if type(self.text)==type(''): #If the text is only text
                f.write(_escape(str(self.text)))
            else:                         #If the text is a spannedtext class
                f.write(str(self.text))
        if self.elements:
            f.write('\t'*level+'</'+self.type+'>\n')
        elif self.text: 
            f.write('</'+self.type+'>\n')
        elif self.cdata:
            f.write('\t'*level+'</'+self.type+'>\n')
        else:
            f.write('/>\n')
            
class tspan(SVGelement):
    """ts=tspan(text='',**args)

    a tspan element can be used for applying formatting to a textsection
    usage:
    ts=tspan('this text is bold')
    ts.attributes['font-weight']='bold'
    st=spannedtext()
    st.addtspan(ts)
    t=text(3,5,st)
    """
    def __init__(self,text=None,**args):
        SVGelement.__init__(self,'tspan',**args)
        if self.text<>None:
            self.text=text
    def __repr__(self):
        s="<tspan"
        for key,value in self.attributes.items():
         s+= ' %s="%s"' % (key,value)
        s+='>'
        s+=self.text
        s+='</tspan>'
        return s
    
class tref(SVGelement):
    """tr=tref(link='',**args)

    a tref element can be used for referencing text by a link to its id.
    usage:
    tr=tref('#linktotext')
    st=spannedtext()
    st.addtref(tr)
    t=text(3,5,st)
    """
    def __init__(self,link,**args):
        SVGelement.__init__(self,'tref',{'xlink:href':link},**args)
    def __repr__(self):
        s="<tref"
        
        for key,value in self.attributes.items():
         s+= ' %s="%s"' % (key,value)
        s+='/>'
        return s
    
class spannedtext:
    """st=spannedtext(textlist=[])

    a spannedtext can be used for text which consists of text, tspan's and tref's
    You can use it to add to a text element or path element. Don't add it directly
    to a svg or a group element.
    usage:
    
    ts=tspan('this text is bold')
    ts.attributes['font-weight']='bold'
    tr=tref('#linktotext')
    tr.attributes['fill']='red'
    st=spannedtext()
    st.addtspan(ts)
    st.addtref(tr)
    st.addtext('This text is not bold')
    t=text(3,5,st)
    """
    def __init__(self,textlist=None):
        if textlist==None:
            self.textlist=[]
        else:
            self.textlist=textlist
    def addtext(self,text=''):
        self.textlist.append(text)
    def addtspan(self,tspan):
        self.textlist.append(tspan)
    def addtref(self,tref):
        self.textlist.append(tref)
    def __repr__(self):
        s=""
        for element in self.textlist:
            s+=str(element)
        return s
    
class rect(SVGelement):
    """r=rect(width,height,x,y,fill,stroke,stroke_width,**args)
    
    a rectangle is defined by a width and height and a xy pair 
    """
    def __init__(self,x=None,y=None,width=None,height=None,fill=None,stroke=None,stroke_width=None,**args):
        if width==None or height==None:
            if width<>None:
                raise ValueError, 'height is required'
            if height<>None:
                raise ValueError, 'width is required'
            else:
                raise ValueError, 'both height and width are required'
        SVGelement.__init__(self,'rect',{'width':width,'height':height},**args)
        if x<>None:
            self.attributes['x']=x
        if y<>None:
            self.attributes['y']=y
        if fill<>None:
            self.attributes['fill']=fill
        if stroke<>None:
            self.attributes['stroke']=stroke
        if stroke_width<>None:
            self.attributes['stroke-width']=stroke_width
            
class ellipse(SVGelement):
    """e=ellipse(rx,ry,x,y,fill,stroke,stroke_width,**args)

    an ellipse is defined as a center and a x and y radius.
    """
    def __init__(self,cx=None,cy=None,rx=None,ry=None,fill=None,stroke=None,stroke_width=None,**args):
        if rx==None or ry== None:
            if rx<>None:
                raise ValueError, 'rx is required'
            if ry<>None:
                raise ValueError, 'ry is required'
            else:
                raise ValueError, 'both rx and ry are required'
        SVGelement.__init__(self,'ellipse',{'rx':rx,'ry':ry},**args)
        if cx<>None:
            self.attributes['cx']=cx
        if cy<>None:
            self.attributes['cy']=cy
        if fill<>None:
            self.attributes['fill']=fill
        if stroke<>None:
            self.attributes['stroke']=stroke
        if stroke_width<>None:
            self.attributes['stroke-width']=stroke_width
        
   
class circle(SVGelement):
    """c=circle(x,y,radius,fill,stroke,stroke_width,**args)

    The circle creates an element using a x, y and radius values eg
    """
    def __init__(self,cx=None,cy=None,r=None,fill=None,stroke=None,stroke_width=None,**args):
        if r==None:
            raise ValueError, 'r is required'
        SVGelement.__init__(self,'circle',{'r':r},**args)
        if cx<>None:
            self.attributes['cx']=cx
        if cy<>None:
            self.attributes['cy']=cy
        if fill<>None:
            self.attributes['fill']=fill
        if stroke<>None:
            self.attributes['stroke']=stroke
        if stroke_width<>None:
            self.attributes['stroke-width']=stroke_width

class point(circle):
    """p=point(x,y,color)
    
    A point is defined as a circle with a size 1 radius. It may be more efficient to use a
    very small rectangle if you use many points because a circle is difficult to render.
    """
    def __init__(self,x,y,fill='black',**args):
        circle.__init__(self,x,y,1,fill,**args)

class line(SVGelement):
    """l=line(x1,y1,x2,y2,stroke,stroke_width,**args)
    
    A line is defined by a begin x,y pair and an end x,y pair
    """
    def __init__(self,x1=None,y1=None,x2=None,y2=None,stroke=None,stroke_width=None,**args):
        SVGelement.__init__(self,'line',**args)
        if x1<>None:
            self.attributes['x1']=x1
        if y1<>None:
            self.attributes['y1']=y1
        if x2<>None:
            self.attributes['x2']=x2
        if y2<>None:
            self.attributes['y2']=y2
        if stroke_width<>None:
            self.attributes['stroke-width']=stroke_width
        if stroke<>None:
            self.attributes['stroke']=stroke
            
class polyline(SVGelement):
    """pl=polyline([[x1,y1],[x2,y2],...],fill,stroke,stroke_width,**args)
    
    a polyline is defined by a list of xy pairs
    """
    def __init__(self,points,fill=None,stroke=None,stroke_width=None,**args):
        SVGelement.__init__(self,'polyline',{'points':_xypointlist(points)},**args)
        if fill<>None:
            self.attributes['fill']=fill
        if stroke_width<>None:
            self.attributes['stroke-width']=stroke_width
        if stroke<>None:
            self.attributes['stroke']=stroke

class polygon(SVGelement):
    """pl=polyline([[x1,y1],[x2,y2],...],fill,stroke,stroke_width,**args)
    
    a polygon is defined by a list of xy pairs
    """
    def __init__(self,points,fill=None,stroke=None,stroke_width=None,**args):
        SVGelement.__init__(self,'polygon',{'points':_xypointlist(points)},**args)
        if fill<>None:
            self.attributes['fill']=fill
        if stroke_width<>None:
            self.attributes['stroke-width']=stroke_width
        if stroke<>None:
            self.attributes['stroke']=stroke

class path(SVGelement):
    """p=path(path,fill,stroke,stroke_width,**args)

    a path is defined by a path object and optional width, stroke and fillcolor
    """
    def __init__(self,pathdata,fill=None,stroke=None,stroke_width=None,id=None,**args):
        SVGelement.__init__(self,'path',{'d':str(pathdata)},**args)
        if stroke<>None:
            self.attributes['stroke']=stroke
        if fill<>None:
            self.attributes['fill']=fill
        if stroke_width<>None:
            self.attributes['stroke-width']=stroke_width
        if id<>None:
            self.attributes['id']=id
        
        
class text(SVGelement):
    """t=text(x,y,text,font_size,font_family,**args)
    
    a text element can bge used for displaying text on the screen
    """
    def __init__(self,x=None,y=None,text=None,font_size=None,font_family=None,text_anchor=None,font_style=None,**args):
        SVGelement.__init__(self,'text',**args)
        if x<>None:
            self.attributes['x']=x
        if y<>None:
            self.attributes['y']=y
        if font_size<>None:
            self.attributes['font-size']=font_size
        if font_family<>None:
            self.attributes['font-family']=font_family
        if font_style<>None:
            self.attributes['font-style']=font_style
        if text<>None:
            self.text=text
        if text_anchor<>None:
            self.attributes['text-anchor']=text_anchor


class textpath(SVGelement):
    """tp=textpath(text,link,**args)

    a textpath places a text on a path which is referenced by a link.   
    """
    def __init__(self,link,text=None,**args):
        SVGelement.__init__(self,'textPath',{'xlink:href':link},**args)
        if text<>None:
            self.text=text

class pattern(SVGelement):
    """p=pattern(x,y,width,height,patternUnits,**args)

    A pattern is used to fill or stroke an object using a pre-defined
    graphic object which can be replicated ("tiled") at fixed intervals
    in x and y to cover the areas to be painted.
    """
    def __init__(self,x=None,y=None,width=None,height=None,patternUnits=None,**args):
        SVGelement.__init__(self,'pattern',**args)
        if x<>None:
            self.attributes['x']=x
        if y<>None:
            self.attributes['y']=y
        if width<>None:
            self.attributes['width']=width
        if height<>None:
            self.attributes['height']=height
        if patternUnits<>None:
            self.attributes['patternUnits']=patternUnits

class title(SVGelement):
    """t=title(text,**args)
    
    a title is a text element. The text is displayed in the title bar
    add at least one to the root svg element
    """
    def __init__(self,text=None,**args):
        SVGelement.__init__(self,'title',**args)
        if text<>None:
            self.text=text

class description(SVGelement):
    """d=description(text,**args)
    
    a description can be added to any element and is used for a tooltip
    Add this element before adding other elements.
    """
    def __init__(self,text=None,**args):
        SVGelement.__init__(self,'desc',**args)
        if text<>None:
            self.text=text

class lineargradient(SVGelement):
    """lg=lineargradient(x1,y1,x2,y2,id,**args)

    defines a lineargradient using two xy pairs.
    stop elements van be added to define the gradient colors.
    """
    def __init__(self,x1=None,y1=None,x2=None,y2=None,id=None,**args):
        SVGelement.__init__(self,'linearGradient',**args)
        if x1<>None:
            self.attributes['x1']=x1
        if y1<>None:
            self.attributes['y1']=y1
        if x2<>None:
            self.attributes['x2']=x2
        if y2<>None:
            self.attributes['y2']=y2
        if id<>None:
            self.attributes['id']=id

class radialgradient(SVGelement):
    """rg=radialgradient(cx,cy,r,fx,fy,id,**args)

    defines a radial gradient using a outer circle which are defined by a cx,cy and r and by using a focalpoint.
    stop elements van be added to define the gradient colors.
    """
    def __init__(self,cx=None,cy=None,r=None,fx=None,fy=None,id=None,**args):
        SVGelement.__init__(self,'radialGradient',**args)
        if cx<>None:
            self.attributes['cx']=cx
        if cy<>None:
            self.attributes['cy']=cy
        if r<>None:
            self.attributes['r']=r
        if fx<>None:
            self.attributes['fx']=fx
        if fy<>None:
            self.attributes['fy']=fy
        if id<>None:
            self.attributes['id']=id
            
class stop(SVGelement):
    """st=stop(offset,stop_color,**args)

    Puts a stop color at the specified radius
    """
    def __init__(self,offset,stop_color=None,**args):
        SVGelement.__init__(self,'stop',{'offset':offset},**args)
        if stop_color<>None:
            self.attributes['stop-color']=stop_color
            
class style(SVGelement):
    """st=style(type,cdata=None,**args)

    Add a CDATA element to this element for defing in line stylesheets etc..
    """
    def __init__(self,type,cdata=None,**args):
        SVGelement.__init__(self,'style',{'type':type},cdata=cdata, **args)
        
            
class image(SVGelement):
    """im=image(url,width,height,x,y,**args)

    adds an image to the drawing. Supported formats are .png, .jpg and .svg.
    """
    def __init__(self,url,x=None,y=None,width=None,height=None,**args):
        if width==None or height==None:
            if width<>None:
                raise ValueError, 'height is required'
            if height<>None:
                raise ValueError, 'width is required'
            else:
                raise ValueError, 'both height and width are required'
        SVGelement.__init__(self,'image',{'xlink:href':url,'width':width,'height':height},**args)
        if x<>None:
            self.attributes['x']=x
        if y<>None:
            self.attributes['y']=y
 
class cursor(SVGelement):
    """c=cursor(url,**args)

    defines a custom cursor for a element or a drawing
    """
    def __init__(self,url,**args):
        SVGelement.__init__(self,'cursor',{'xlink:href':url},**args)

    
class marker(SVGelement):
    """m=marker(id,viewbox,refX,refY,markerWidth,markerHeight,**args)
    
    defines a marker which can be used as an endpoint for a line or other pathtypes
    add an element to it which should be used as a marker.
    """
    def __init__(self,id=None,viewBox=None,refx=None,refy=None,markerWidth=None,markerHeight=None,**args):
        SVGelement.__init__(self,'marker',**args)
        if id<>None:
            self.attributes['id']=id
        if viewBox<>None:
            self.attributes['viewBox']=_viewboxlist(viewBox)
        if refx<>None:
            self.attributes['refX']=refx
        if refy<>None:
            self.attributes['refY']=refy
        if markerWidth<>None:
            self.attributes['markerWidth']=markerWidth
        if markerHeight<>None:
            self.attributes['markerHeight']=markerHeight
        
class group(SVGelement):
    """g=group(id,**args)
    
    a group is defined by an id and is used to contain elements
    g.addElement(SVGelement)
    """
    def __init__(self,id=None,**args):
        SVGelement.__init__(self,'g',**args)
        if id<>None:
            self.attributes['id']=id

class symbol(SVGelement):
    """sy=symbol(id,viewbox,**args)

    defines a symbol which can be used on different places in your graph using
    the use element. A symbol is not rendered but you can use 'use' elements to
    display it by referencing its id.
    sy.addElement(SVGelement)
    """
    
    def __init__(self,id=None,viewBox=None,**args):
        SVGelement.__init__(self,'symbol',**args)
        if id<>None:
            self.attributes['id']=id
        if viewBox<>None:
            self.attributes['viewBox']=_viewboxlist(viewBox)
            
class defs(SVGelement):
    """d=defs(``**args``)

    container for defining elements
    """
    def __init__(self,**args):
        SVGelement.__init__(self,'defs',**args)

class switch(SVGelement):
    """sw=switch(``**args``)

    Elements added to a switch element which are "switched" by the attributes
    requiredFeatures, requiredExtensions and systemLanguage.
    Refer to the SVG specification for details.
    """
    def __init__(self,**args):
        SVGelement.__init__(self,'switch',**args)

        
class use(SVGelement):
    """u=use(link,x,y,width,height,``**args``)
    
    references a symbol by linking to its id and its position, height and width
    """
    def __init__(self,link,x=None,y=None,width=None,height=None,**args):
        SVGelement.__init__(self,'use',{'xlink:href':link},**args)
        if x<>None:
            self.attributes['x']=x
        if y<>None:
            self.attributes['y']=y

        if width<>None:
            self.attributes['width']=width
        if height<>None:
            self.attributes['height']=height
            
            
class link(SVGelement):
    """a=link(url,``**args``)

    a link  is defined by a hyperlink. add elements which have to be linked
    a.addElement(SVGelement)
    """
    def __init__(self,link='',**args):
        SVGelement.__init__(self,'a',{'xlink:href':link},**args)
        
class view(SVGelement):
    """v=view(id,``**args``)

    a view can be used to create a view with different attributes"""
    def __init__(self,id=None,**args):
        SVGelement.__init__(self,'view',**args)
        if id<>None:
            self.attributes['id']=id

class script(SVGelement):
    """sc=script(type,type,cdata,``**args``)

    adds a script element which contains CDATA to the SVG drawing

    """
    def __init__(self,type,cdata=None,**args):
        SVGelement.__init__(self,'script',{'type':type},cdata=cdata,**args)
        
class animate(SVGelement):
    """an=animate(attribute,from,to,during,``**args``)

    animates an attribute.    
    """
    def __init__(self,attribute,fr=None,to=None,dur=None,**args):
        SVGelement.__init__(self,'animate',{'attributeName':attribute},**args)
        if fr<>None:
            self.attributes['from']=fr
        if to<>None:
            self.attributes['to']=to
        if dur<>None:
            self.attributes['dur']=dur
        
class animateMotion(SVGelement):
    """an=animateMotion(pathdata,dur,``**args``)

    animates a SVGelement over the given path in dur seconds
    """
    def __init__(self,pathdata,dur,**args):
        SVGelement.__init__(self,'animateMotion',**args)
        if pathdata<>None:
            self.attributes['path']=str(pathdata)
        if dur<>None:
            self.attributes['dur']=dur

class animateTransform(SVGelement):
    """antr=animateTransform(type,from,to,dur,``**args``)
    
    transform an element from and to a value.
    """
    def __init__(self,type=None,fr=None,to=None,dur=None,**args):
        SVGelement.__init__(self,'animateTransform',{'attributeName':'transform'},**args)
        #As far as I know the attributeName is always transform
        if type<>None:
            self.attributes['type']=type
        if fr<>None:
            self.attributes['from']=fr
        if to<>None:
            self.attributes['to']=to
        if dur<>None:
            self.attributes['dur']=dur
class animateColor(SVGelement):
    """ac=animateColor(attribute,type,from,to,dur,``**args``)

    Animates the color of a element
    """
    def __init__(self,attribute,type=None,fr=None,to=None,dur=None,**args):
        SVGelement.__init__(self,'animateColor',{'attributeName':attribute},**args)
        if type<>None:
            self.attributes['type']=type
        if fr<>None:
            self.attributes['from']=fr
        if to<>None:
            self.attributes['to']=to
        if dur<>None:
            self.attributes['dur']=dur        
class set(SVGelement):
    """st=set(attribute,to,during,``**args``)
    
    sets an attribute to a value for a
    """
    def __init__(self,attribute,to=None,dur=None,**args):
        SVGelement.__init__(self,'set',{'attributeName':attribute},**args)
        if to<>None:
            self.attributes['to']=to
        if dur<>None:
            self.attributes['dur']=dur


            
class svg(SVGelement):
    """s=svg(viewbox,width,height,``**args``)
    
    a svg or element is the root of a drawing add all elements to a svg element.
    You can have different svg elements in one svg file
    s.addElement(SVGelement)

    eg
    d=drawing()
    s=svg((0,0,100,100),'100%','100%')
    c=circle(50,50,20)
    s.addElement(c)
    d.setSVG(s)
    d.toXml()
    """
    def __init__(self,viewBox=None, width=None, height=None,**args):
        SVGelement.__init__(self,'svg',**args)
        if viewBox<>None:
            self.attributes['viewBox']=_viewboxlist(viewBox)
        if width<>None:
            self.attributes['width']=width
        if height<>None:
            self.attributes['height']=height
        self.namespace="http://www.w3.org/2000/svg"
        
class drawing:
    """d=drawing()

    this is the actual SVG document. It needs a svg element as a root.
    Use the addSVG method to set the svg to the root. Use the toXml method to write the SVG
    source to the screen or to a file
    d=drawing()
    d.addSVG(svg)
    d.toXml(optionalfilename)
    """

    def __init__(self):
        self.svg=None
    def setSVG(self,svg):
        self.svg=svg
        #Voeg een element toe aan de grafiek toe.
    if use_dom_implementation==0:      
        def toXml(self, filename='',compress=False):
            import cStringIO
            xml=cStringIO.StringIO()
            xml.write("<?xml version='1.0' encoding='UTF-8'?>\n")
            xml.write("<!DOCTYPE svg PUBLIC \"-//W3C//DTD SVG 1.0//EN\" \"http://www.w3.org/TR/2001/REC-SVG-20010904/DTD/svg10.dtd \">\n")      
            self.svg.toXml(0,xml)
            if not filename:
                if compress:
                    import gzip
                    f=cStringIO.StringIO()
                    zf=gzip.GzipFile(fileobj=f,mode='wb')
                    zf.write(xml.getvalue())
                    zf.close()
                    f.seek(0)
                    return f.read()
                else:
                    return xml.getvalue()
            else:
                if filename[-4:]=='svgz':
                    import gzip
                    f=gzip.GzipFile(filename=filename,mode="wb", compresslevel=9)
                    f.write(xml.getvalue())
                    f.close()
                else:
                    f=file(filename,'w')
                    f.write(xml.getvalue())
                    f.close()

    else:
        def toXml(self,filename='',compress=False):
            """drawing.toXml()        ---->to the screen
            drawing.toXml(filename)---->to the file
            writes a svg drawing to the screen or to a file
            compresses if filename ends with svgz or if compress is true
            """
            doctype = implementation.createDocumentType('svg',"-//W3C//DTD SVG 1.0//EN""",'http://www.w3.org/TR/2001/REC-SVG-20010904/DTD/svg10.dtd ')
        
            global root
            #root is defined global so it can be used by the appender. Its also possible to use it as an arugument but
            #that is a bit messy.
            root=implementation.createDocument(None,None,doctype)
            #Create the xml document.
            global appender
            def appender(element,elementroot):
                """This recursive function appends elements to an element and sets the attributes
                and type. It stops when alle elements have been appended"""
                if element.namespace:
                    e=root.createElementNS(element.namespace,element.type)
                else:
                    e=root.createElement(element.type)
                if element.text:
                    textnode=root.createTextNode(element.text)
                    e.appendChild(textnode)
                for attribute in element.attributes.keys():   #in element.attributes is supported from python 2.2
                    e.setAttribute(attribute,str(element.attributes[attribute]))
                if element.elements:
                    for el in element.elements:
                        e=appender(el,e)
                elementroot.appendChild(e)
                return elementroot
            root=appender(self.svg,root)
            if not filename:
                import cStringIO
                xml=cStringIO.StringIO()
                PrettyPrint(root,xml)
                if compress:
                    import gzip
                    f=cStringIO.StringIO()
                    zf=gzip.GzipFile(fileobj=f,mode='wb')
                    zf.write(xml.getvalue())
                    zf.close()
                    f.seek(0)
                    return f.read()
                else:
                    return xml.getvalue()
            else:
                try:
                    if filename[-4:]=='svgz':
                        import gzip
                        import cStringIO
                        xml=cStringIO.StringIO()
                        PrettyPrint(root,xml)
                        f=gzip.GzipFile(filename=filename,mode='wb',compresslevel=9)
                        f.write(xml.getvalue())
                        f.close()
                    else:
                        f=open(filename,'w')
                        PrettyPrint(root,f)
                        f.close()
                except:
                    print "Cannot write SVG file: " + filename
    def validate(self):
        try:
            import xml.parsers.xmlproc.xmlval
        except:
            raise exceptions.ImportError,'PyXml is required for validating SVG'
        svg=self.toXml()
        xv=xml.parsers.xmlproc.xmlval.XMLValidator()
        try:
            xv.feed(svg)
        except:
            raise "SVG is not well formed, see messages above"
        else:
            print "SVG well formed"
if __name__=='__main__':

    
    d=drawing()
    s=svg((0,0,100,100))
    r=rect(-100,-100,300,300,'cyan')
    s.addElement(r)
    
    t=title('SVGdraw Demo')
    s.addElement(t)
    g=group('animations')
    e=ellipse(0,0,5,2)
    g.addElement(e)
    c=circle(0,0,1,'red')
    g.addElement(c)
    pd=pathdata(0,-10)
    for i in range(6):
        pd.relsmbezier(10,5,0,10)
        pd.relsmbezier(-10,5,0,10)
    an=animateMotion(pd,10)
    an.attributes['rotate']='auto-reverse'
    an.attributes['repeatCount']="indefinite"
    g.addElement(an)
    s.addElement(g)
    for i in range(20,120,20):
        u=use('#animations',i,0)
        s.addElement(u)
    for i in range(0,120,20):
        for j in range(5,105,10):
            c=circle(i,j,1,'red','black',.5)
            s.addElement(c)
    d.setSVG(s)
     
    print d.toXml()




################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Tildon Grant Belgard
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
"""
==================================================================
Combine hard masked + unmasked genomes to give soft masked genome
==================================================================

:Author: David Sims
:Release: $Id$
:Date: |today|
:Tags: Python

This program takes two input files: a hard masked genome and a complementary unmasked genome ans outputs a soft masked genome.


Usage
=====

softmask -u <unmasked.fasta> -h <hard_masked.fasta> -o <outfile.fasta>


Code
====

"""

# load modules

import logging as L
import CGAT.Experiment as E
import sys
import os
import re
import shutil
import itertools
import math
import glob
import time
import gzip
import collections
import random
import optparse
import CGAT.GTF as GTF
import CGAT.IOTools as IOTools
import CGAT.IndexedFasta as IndexedFasta

def main( argv = None ):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if not argv: argv = sys.argv

    # setup command line parser
    parser = E.OptionParser( version = "%prog version: $Id: snp2table.py 2861 2010-02-23 17:36:32Z andreas $", usage = globals()["__doc__"] )

    parser.add_option("-u", "--unmasked", dest="unmasked_genome_file", type="string",
                      help="filename with unmasked genome [default=%default]."  )
    parser.add_option("-r", "--hard-masked", dest="hard_masked_genome_file", type="string",
                      help="filename with hard masked genome [default=%default]."  )
    parser.add_option("-o", "--output", dest="output_file", type="string",
                      help="output filename  [default=%default]."  )

    parser.set_defaults(
        unmasked_genome_file = None,
        hard_masked_genome_file = None,
        output_file = None,
        )

    ## add common options (-h/--help, ...) and parse command line 
    (options, args) = E.Start( parser, argv = argv )

    ninput, nunchanged, nchanged = 0, 0, 0

    # Open input files
    E.info("Opening input file: %s" % options.unmasked_genome_file)
    unmask = open(options.unmasked_genome_file, "r" )
    E.info("Opening input file: %s" % options.hard_masked_genome_file)
    hardmask = open(options.hard_masked_genome_file, "r" )

    # Open output file
    softmask = open(options.output_file, "w")

    # Loop over input files and convert to soft clipped
    for line in hardmask:
        unmask_line = unmask.readline()
        ninput = ninput + 1

        if line.startswith( ">" ) or line==unmask_line:
            softmask.write( line )
            nunchanged = nunchanged + 1
        else:
            softmask_line = ""
            for i, c in enumerate(line):
                if c=="N":
                    softmask_line = softmask_line + unmask_line[i].lower()
                else:
                    softmask_line = softmask_line + c
            softmask.write( softmask_line )
            nchanged = nchanged + 1
        

    # Close all files
    unmask.close()
    hardmask.close()
    softmask.close()
                        
    # Report statistics
    E.info( "ninput=%i, nunchanged=%i, nchanged=%i" % (ninput,nunchanged,nchanged) )

    ## write footer and output benchmark information.
    E.Stop()

if __name__ == "__main__":
    sys.exit( main( sys.argv) )




################################################################################
#   Gene prediction pipeline 
#
#   $Id: tree2matrix.py 2782 2009-09-10 11:40:29Z andreas $
#
#   Copyright (C) 2004 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
"""
tree2matrix.py - convert a tree to a distance matrix
====================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

convert one or more trees into distance matrix. The nodes
in the tree between distances are output is defined by
the --pairs option. Possible values are:

+--------------------+----------------------------------------------------------------------+
|                    |compute distances between all nodes in the tree                       |
|all                 |                                                                      |
|                    |                                                                      |
+--------------------+----------------------------------------------------------------------+
|leaves              |compute distance between all leaves in the tree                       |
|                    |                                                                      |
|                    |                                                                      |
+--------------------+----------------------------------------------------------------------+
|branches            |computed distances between each branches                              |
|                    |                                                                      |
|                    |                                                                      |
+--------------------+----------------------------------------------------------------------+
|terminals           |get distance of each leave to its parent                              |
|                    |                                                                      |
+--------------------+----------------------------------------------------------------------+
|lineage             |get distance of each leaf to the next speciation node                 |
|                    |                                                                      |
|                    |                                                                      |
+--------------------+----------------------------------------------------------------------+
|between-species     |get distance between different species, excluding same species nodes  |
|                    |                                                                      |
|                    |                                                                      |
|                    |                                                                      |
+--------------------+----------------------------------------------------------------------+

Usage
-----

Example::

   python <script_name>.py --help

Type::

   python <script_name>.py --help

for command line help.

Documentation
-------------

Code
----

""" 

import os
import sys
import string
import re
import getopt
import time
import optparse
import math
import tempfile
import subprocess

from types import *
""" program $Id: tree2matrix.py 2782 2009-09-10 11:40:29Z andreas $

"""
import CGAT.Experiment as E
import CGAT.TreeTools as TreeTools

def TranslateNode( node, tree, terminals, options ):
    if options.do_translate:
        return options.separator.join( sorted(TreeTools.GetLeaves( tree, node )) )
    elif node in terminals:
        return tree.node( node ).data.taxon
    else:
        return str(node)

if __name__ == "__main__":

    parser = E.OptionParser( version = "%prog version: $Id: tree2matrix.py 2782 2009-09-10 11:40:29Z andreas $")

    parser.add_option("-f", "--format", dest="format", type="string",
                      help="number format to use."  )
    parser.add_option("-g", "--graph", dest="to_graph", action="store_true",
                      help="convert tree(s) to graph(s)."  )
    parser.add_option("-a", "--table", dest="to_table", action="store_true",
                      help="convert tree(s) to table."  )
    parser.add_option("-t", "--translate", dest="do_translate", action="store_true",
                      help="translate internal nodes to clades."  )
    parser.add_option( "--output-pattern", dest="output_filename_pattern", type="string",
                       help="pattern for output file if there are multiple trees in the file.""")
    parser.add_option( "--pairs", dest="pairs", type="choice",
                       choices=("all", "leaves", "branches", "terminals", "lineage", "between-species" ),
                       help="choose pairs of nodes to output.""")
    parser.add_option( "--species", dest="species", type="string",
                       help="comma separated list of species that are considered. All others are ignored." )


    parser.set_defaults(
        format = "%6.4f",
        to_graph = False,
        to_table = False,
        do_translation = False,
        separator = ":",
        do_all_on_all = False,
        do_branches = False,
        do_terminals = False,
        output_filename_pattern = None,
        pairs = "branches",
        species = None,
        regex_species = ("^([^|]+)" ),
        )

    (options, args) = E.Start( parser, add_pipe_options = True )

    if options.species: options.species = options.species.split(",")
    nexus = TreeTools.Newick2Nexus( sys.stdin )

    if options.loglevel >= 1:
        options.stdlog.write( "# read %i trees from stdin.\n" % len(nexus.trees))

    ntree = 0
    outfile = None

    ## The table is a hash of lists
    table = {}

    extract_species = lambda x: re.search( options.regex_species, x).groups()[0]

    for tree in nexus.trees:

        if len(nexus.trees) == 1:
            outfile = options.stdout
        elif options.output_filename_pattern:
            ntree += 1
            if outfile != None: outfile.close()            
            outfile = open( options.output_filename_pattern % ntree, "w" )
        else:
            outfile = options.stdout
            
        ## prune tree, if an explicit species list is given
        if options.species:
            species = set(options.species)
            terminals = tree.get_terminals()
            for x in terminals:
                taxon = tree.node(x).data.taxon
                if extract_species(taxon) not in species:
                    tree.prune( taxon )

        ## define node list
        terminals = tree.get_terminals()
        set_terminals = set(terminals)
        node_list = []
        
        if options.pairs == "all":
            nodes = TreeTools.GetAllNodes(tree)
            for x in range(len(nodes)):
                for y in range(0,x):
                    node_list.append( (nodes[x], nodes[y]) )
        elif options.pairs == "terminals":
            for x in terminals:
                node_list.append( (x, tree.node(x).prev) )
        elif options.pairs == "leaves":
            nodes = terminals
            for x in range(len(nodes)):
                for y in range(0,x):
                    node_list.append( (nodes[x], nodes[y]) )
        elif options.pairs == "branches":
            nodes = TreeTools.GetAllNodes(tree)
            for x in range(len(nodes)):
                if tree.node(x).prev:
                    node_list.append( (x, tree.node(x).prev) )
        elif options.pairs == "between-species":
            nodes = terminals
            for x in range(len(nodes)):
                for y in range(0,x):
                    s1 = extract_species( tree.node(nodes[x]).data.taxon) 
                    s2 = extract_species( tree.node(nodes[y]).data.taxon) 
                    if s1 != s2:
                        node_list.append( (nodes[x], nodes[y]) )

        elif options.pairs == "lineage":
            raise "not implemented."
        
        if options.to_graph:
            outfile.write( "node1\tnode2\tdistance\n" )
            
            links = TreeTools.Tree2Graph( tree )
            for n1, n2, weight in links:

                node1 = TranslateNode( n1, tree, set_terminals, options )
                node2 = TranslateNode( n2, tree, set_terminals, options )

                if node1 > node2: node1, node2 = node2, node1
                outfile.write( "%s\t%s\t%s\n" % (
                    node1, node2,
                    options.format % weight ) )

        elif options.to_table:
            
            if options.do_all_on_all:
                nodes = TreeTools.GetAllNodes(tree)
            else:
                nodes = terminals

            for n1, n2 in node_list:
                
                node1 = TranslateNode( n1, tree, set_terminals, options )
                node2 = TranslateNode( n2, tree, set_terminals, options )

                if node1 > node2: node1, node2 = node2, node1

                if options.do_terminals:
                    key = "%s" % node2
                else:
                    key = "%s-%s" % (node1, node2)
                    
                if key not in table: table[key] = []

                table[key].append( options.format % tree.distance( n1, n2 ) )
        else:
            outfile.write( "node1\tnode2\tdistance\n" )
            
            for n1, n2 in node_list:
                node1 = TranslateNode( n1, tree, set_terminals, options )
                node2 = TranslateNode( n2, tree, set_terminals, options )
                
                if node1 > node2: node1, node2 = node2, node1
                
                outfile.write( "%s\t%s\t%s\n" % ( \
                        node1, node2, 
                        options.format % tree.distance( n1, n2 )))

    if options.to_table:
        outfile = sys.stdout
        outfile.write("branch\t%s\n" % ("\t".join(map(str, range(0,len(nexus.trees))))))
                                                      
        for key, values in table.items():
            outfile.write( "%s\t%s\n" % (key, "\t".join(values)))
        
    if outfile != sys.stdout:
        outfile.close()
    
    E.Stop()



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
barplotGo.py
=============================================

:Author: 
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This script takes the output from GO.py (section = results) and produces a barplot of the 
enrichment results. The plotting is performed using Rpy.

Usage
-----

Example::

   python barplotGo.py --help

Type::

   python barplotGo.py --help

for command line help.

Documentation
-------------

Code
----

'''

import os
import sys
import re
import optparse

import CGAT.Experiment as E
from rpy2.robjects import r as R


def main( argv = None ):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if not argv: argv = sys.argv

    # setup command line parser
    parser = E.OptionParser( version = "%prog version: $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $", 
                                    usage = globals()["__doc__"] )

    parser.add_option("-i", "--infile", dest="infile", type="string",
                      help="supply infile name"  )
    parser.add_option("-o", "--outfile", dest="outfile", type="string",
                      help="supply outfile name"  )
    parser.add_option("-t", "--height", dest="height", type="int",
                      help="height of plotting window"  )
    parser.add_option("-w", "--width", dest="width", type="int",
                      help="number of bars"  )
    parser.add_option("-n", "--number", dest="number", type="int",
                      help="width of plotting window"  )
    parser.add_option("-c", "--colour", dest="colour", type="choice"
                      , choices=("red", "purple", "blue", "grey", "black", "pink", "green")
                      , help="colour choices"  )
    parser.add_option("--side", dest="side", action="store_true"
                      , help="set to draw sideways barplot"  )
    parser.add_option("-m", "--marigins", dest="margins", type="string"
                      , help="set margins - follows R convention for mai"  )
    parser.add_option("--cexn", dest="cex_names", type="float"
                      , help="set the size of the names on the barplot"  )
    parser.add_option("--cexa", dest="cex_axis", type="float"
                      , help="set the size of the axis labels on the barplot"  )

    parser.set_defaults(
        height = 500
        , width = 500
        , number = 5
        , colour = "blue"
        , margins = "1,1,1,1"
        , cex_names = 1
        , cex_axis = 1)

    ## add common options (-h/--help, ...) and parse command line 
    (options, args) = E.Start( parser, argv = argv )

    # don't plot more than 20
    if options.number > 20:
        sys.exit(0)

    inf = options.infile
    outf = options.outfile

    R('''data <- read.table("%s", sep = "\t", header = T, stringsAsFactors = F)''' % inf)
    R('''pdf("%s", height = %i, width = %i)''' % (outf, options.height, options.width) )

    R('''ratio <- as.matrix(data$ratio[c(1:%i)])''' % options.number)
    R('''par(mai = c(%s))''' % options.margins)
    if options.side:
        horiz = "horiz = T"
    else:
        horiz = "horiz = F"
    R('''barplot(ratio
                     , beside = T
                     , names = data$description[c(1:%i)]
                     , col = "%s"
                     , las = 2
                     , %s
                     , cex.names = %f
                     , cex.axis = %f)''' % (options.number, options.colour, horiz, options.cex_names, options.cex_axis))
    R["dev.off"]()

    ## write footer and output benchmark information.
    E.Stop()

if __name__ == "__main__":
    sys.exit( main( sys.argv) )





################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
gtf2tsv.py - convert gtf file to a tab-separated table
======================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

convert gtf formatted file to tab-separated table with column headers for 
table import.

Note that coordinates are converted to 0-based open/closed notation (all on
the forward strand).

If -a/--attributes is set, attributes are converted into separate columns.

Usage
-----

Example::

   python gtf2tsv.py < in.gtf > out.tsv

Type::

   python gtf2tsv.py --help

for command line help.

Command line options
---------------------

'''
import os
import sys
import string
import re
import optparse
import CGAT.GTF as GTF
import CGAT.Experiment as E

def main():
    '''
    main function
    '''
    parser = E.OptionParser( version = "%prog version: $Id: gtf2tsv.py 2887 2010-04-07 08:48:04Z andreas $", usage = globals()["__doc__"])

    parser.add_option( "-o", "--only-attributes", dest="only_attributes", action="store_true",
                       help="output attributes as separate columns [default=%default]." )
    parser.add_option( "-f", "--full", dest="full", action="store_true",
                       help="output attributes as separate columns [default=%default]." )
    parser.add_option( "-i", "--invert", dest="invert", action = "store_true",
                       help="convert tab-separated table back to gtf [default=%default]." ) 
    parser.add_option( "-m", "--map", dest="map", type="choice",
                       choices=("transcript2gene", "peptide2gene", "peptide2transcript"),
                       help="output a map mapping transcripts to genes [default=%default]." ) 



    parser.set_defaults(
        only_attributes = False,
        full = False,
        invert = False,
        map = None,
        )


    (options,args) = E.Start( parser )

    if options.full:

        # output full table with column for each attribute
        attributes = set()
        data = []
        for gtf in GTF.iterator( options.stdin ):
            data.append(gtf)
            attributes = attributes.union( set(gtf.keys()) )

        # remove gene_id and transcript_id, as they are used
        # explicitely later
        attributes.difference_update( ["gene_id", "transcript_id"] ) 
            
        attributes = sorted(list(attributes))

        if options.only_attributes:
            header = [ "gene_id", "transcript_id"] + attributes
        else:
            header = [ "contig","source","feature","start","end","score","strand","frame","gene_id",
                       "transcript_id", ] + attributes

        options.stdout.write("\t".join( header ) + "\n" )

        if options.only_attributes:
            for gtf in data:
                options.stdout.write( "\t".join( map(str, (gtf.gene_id,
                                                           gtf.transcript_id,
                                                           ))))
                for a in attributes:
                    if a in ("gene_id", "transcript_id"): continue
                    try:
                        val = getattr( gtf, a )
                    except AttributeError:
                        val = ""
                    options.stdout.write( "\t%s" % val )
                options.stdout.write("\n")
        else:
            for gtf in data:
                options.stdout.write( "\t".join( map(str, (gtf.contig,
                                                           gtf.source,
                                                           gtf.feature,
                                                           gtf.start,
                                                           gtf.end,
                                                           gtf.score,
                                                           gtf.strand,
                                                           gtf.frame,
                                                           gtf.gene_id,
                                                           gtf.transcript_id,
                                                           ))))
                for a in attributes:
                    try:
                        val = getattr( gtf, a )
                    except AttributeError:
                        val = ""
                    options.stdout.write( "\t%s" % val )
                options.stdout.write("\n")
                
    elif options.invert:

        gtf = GTF.Entry()
        header = None
        for line in options.stdin:
            if line.startswith("#"): continue
            data = line[:-1].split("\t")
            if not header:
                header = data
                map_header2column = dict( [(y,x) for x,y in enumerate( header )] )
                continue
            
            # fill gtf entry with data
            try:
                gtf.contig = data[map_header2column["contig"]]
                gtf.source = data[map_header2column["source"]]
                gtf.feature = data[map_header2column["feature"]]
                # subtract -1 to start for 0-based coordinates
                gtf.start = int(data[map_header2column["start"]])
                gtf.end = int(data[map_header2column["end"]])
                gtf.score = data[map_header2column["score"]]
                gtf.strand = data[map_header2column["strand"]]
                gtf.frame = data[map_header2column["frame"]]
                gtf.gene_id = data[map_header2column["gene_id"]]
                gtf.transcript_id = data[map_header2column["transcript_id"]]
                gtf.parseInfo( data[map_header2column["attributes"]], line )
            except KeyError, msg:
                raise KeyError( "incomplete entry %s: %s: %s" % (str(data), str(map_header2column), msg))
            # output gtf entry in gtf format
            options.stdout.write( "%s\n" % str(gtf) )

    elif options.map:
        
        if options.map == "transcript2gene":
            fr = lambda x: x.transcript_id
            to = lambda x: x.gene_id
            options.stdout.write("transcript_id\tgene_id\n" )            
        elif options.map == "peptide2gene":
            fr = lambda x: x.protein_id
            to = lambda x: x.gene_id
            options.stdout.write("peptide_id\tgene_id\n" )            
        elif options.map == "peptide2transcript":
            fr = lambda x: x.protein_id
            to = lambda x: x.transcript_id
            options.stdout.write("peptide_id\ttranscript_id\n" )            
            
        map_fr2to = {}
        for gtf in GTF.iterator( options.stdin ):
            try:
                map_fr2to[fr(gtf)] = to(gtf)
            except AttributeError:
                pass

        for x,y in sorted(map_fr2to.iteritems()):
            options.stdout.write("%s\t%s\n" % (x,y) )
    else:
        header = ( "contig","source","feature","start","end","score","strand","frame","gene_id","transcript_id","attributes"  )
        options.stdout.write("\t".join( header ) + "\n" )

        for gtf in GTF.iterator( options.stdin ):
            
            attributes = []
            for a in gtf.keys():
                if a in ("gene_id", "transcript_id"): continue
                attributes.append( '%s %s' % (a, GTF.quote(gtf[a])) )

            attributes = "; ".join( attributes )
            
            options.stdout.write( "\t".join( map(str, (gtf.contig,
                                                       gtf.source,
                                                       gtf.feature,
                                                       gtf.start,
                                                       gtf.end,
                                                       GTF.toDot( gtf.score ),
                                                       gtf.strand,
                                                       gtf.frame,
                                                       gtf.gene_id,
                                                       gtf.transcript_id,
                                                       attributes,
                                                       ) ) ) + "\n" )
    E.Stop()

if __name__ == '__main__':
    sys.exit(main())





################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id: script_template.py 2871 2010-03-03 10:20:44Z andreas $
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
bam2species_map.py
=============================================

:Author: Nick Ilott
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

Produce a mapping txt file between contigs a species based
on aligned reads.

Usage
-----

Example::

   python bam2species_map.py --help

Type::

   python bam2species_map.py --help

for command line help.

Documentation
-------------

This script would be used as a pre-step to using contigs2random_sample.py. It provides
a mapping between contigs and species that are represented in those contigs i.e. in 
a metagenomic simulation study the majority species for a contig will be returned with
the contig.

Code
----

'''

import os
import sys
import re
import optparse

import CGAT.Experiment as E
import CGAT.IOTools as IOTools
import pysam
import collections
import CGAT.FastaIterator as FastaIterator

def main( argv = None ):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if not argv: argv = sys.argv

    # setup command line parser
    parser = optparse.OptionParser( version = "%prog version: $Id: script_template.py 2871 2010-03-03 10:20:44Z andreas $", 
                                    usage = globals()["__doc__"] )

    parser.add_option("-b", "--bamfile", dest="bamfile", type="string",
                      help="supply bam file"  )

    ## add common options (-h/--help, ...) and parse command line 
    (options, args) = E.Start( parser, argv = argv )

    # read in contigs
    E.info("reading in contig file")
    contigs = {}
    for fasta in FastaIterator.iterate(options.stdin):
        contigs[fasta.title] = (1, len(fasta.sequence) -1)
    E.info("read %i contigs" % len(contigs.keys()))

    # read in bamfile
    E.info("reading bam file")
    samfile = pysam.Samfile(options.bamfile)

    E.info("iterating over contigs")
    c = 0
    for contig, coords in contigs.iteritems():
        coords = list(coords)

        #################################
        # NB this is specific for my data!
        contig = contig.split(" ")[0]
        #################################

        species_counts = collections.defaultdict(int)
        for alignment in samfile.fetch(contig, coords[0], coords[1]):
            species_id = alignment.qname.split("|")[1]
            species_counts[species_id] += 1

        # at the moment ignore if there are no counts
        if len(species_counts.values()) == 0: 
            E.warn("no reads map to %s" % contig)
            continue

        for species, count in species_counts.iteritems():
            if species_counts[species] == max(species_counts.values()): 
                top_dog = species
                c += 1
                break
        E.info("species %s assigned to contig number %i" % (top_dog, c))
        options.stdout.write("%s\t%s\n" % (contig, top_dog))

    ## write footer and output benchmark information.
    E.Stop()

if __name__ == "__main__":
    sys.exit( main( sys.argv) )



"""MAST.py - Parser for MAST files
==================================

:Author: 
:Release: $Id$
:Date: |today|
:Tags: Python

As of biopython 1.5.6, the MAST parser is broken.
"""

import string, sys, re, types, math, collections

import numpy

class MAST:
    # number of matches
    def __init__(self): 
        self.motifs = []
        self.matches = []

class Motif: pass

class Match:
    """a MAST entry.
    """
    header = "\t".join( ("id", "description", "length", "pvalue", "evalue", "nmatches", "diagram", "start", "end", "positions", "motifs", "directions") )

    def __init__(self):
        pass

    def __str__(self):
        return "\t".join( map(str,
                              (self.id,
                               self.description,
                               self.length,
                               self.pvalue,
                               self.evalue,
                               self.nmotifs,
                               self.diagram,
                               self.start,
                               self.end,
                               ",".join(map(str,self.positions)),
                               ",".join(map(str,self.motifs)),
                               ",".join(self.directions)) ) )
        

def parse( infile ):
    '''parse verbose MAST output.'''

    line = None
    for line in infile:
        if line.startswith( "DATABASE AND MOTIFS"): 
            break
    
    mast = MAST()
    # empty input
    if not line: return mast
        
    keep = False
    for line in infile:
        l = line.strip()
        if l.startswith("MOTIF WIDTH"): 
            keep = True
            continue
        if not keep: continue
        if keep:
            if l.startswith("-"): continue
            if not l: break
            motif = Motif()
            motif.id, motif.length, motif.sequence = re.split("\s+", l)
            motif.length = int(motif.length)
            mast.motifs.append( motif )

    for line in infile:
        if line.startswith( "SECTION III"): 
            break

    assert line, "could not find tag 'SECTION III'"
    
    def __myiter( infile ):
        keep = False
        do_yield = False
        for line in infile:
            if line.startswith("*"): continue
            if keep:
                l = line[:-1].strip()
                if do_yield:
                    # collect continuation of DIAGRAM line
                    if len(line) > 2:
                        block[-1] += l
                        continue
                    else:
                        yield block
                        keep = False
                        do_yield = False

                block.append( l )
                # start end of block (DIAGRAM might be multiline)
                if l.startswith( "DIAGRAM" ):
                    do_yield = True
            else:
                if not re.match( "[\d\s]", line):
                    # start of new block
                    keep = True
                    block = [line[:-1].strip()]
        
    for block in __myiter(infile ):
        assert len(block) == 4, "invalid block: %s" % "".join( block )
        match = Match()
        match.id = block[0]
        match.description = block[1]
        match.length, match.pvalue, match.evalue = re.match( "LENGTH =\s*(\d+)\s*COMBINED P-VALUE =\s*(\S+)\s*E-VALUE =\s*(\S+)", block[2]).groups()
        match.length, match.pvalue, match.evalue = \
            int(match.length), float(match.pvalue), float(match.evalue)
        match.diagram =  re.match( "DIAGRAM:\s+(\S+)", block[3] ).groups()[0]
        parts = match.diagram.split("_")
        match.nmotifs =  len(parts) // 2
        match.positions, match.directions, match.motifs = [], [], []
        match.start = ""
        match.end = ""
        if match.nmotifs > 0:
            pos = 0
            last_end = 0
            try:
                for part in parts:
                    is_motif = part[0] in "[<"
                    if is_motif:
                        code = part[1:-1]
                        match.positions.append( pos )
                        match.directions.append( code[0] )
                        motif_id = int(code[1:])
                        match.motifs.append( motif_id )
                        pos += mast.motifs[motif_id-1].length
                        last_end = pos
                    else:
                        pos += int( part )
            except ValueError:
                raise ValueError( "ParsingError for motif diagram %s" % match.diagram)

            match.start = match.positions[0]
            assert last_end > 0
            match.end = last_end

        mast.matches.append( match )
                  
    return mast

def frequencies2logodds( counts, background_frequencies = None ): 
    '''write a motif from *counts* to outfile.

    Counts should be a numpy matrix with *nalphabet* columns and *motif_width* rows.
    '''

    motif_width, nalphabet = counts.shape

    if background_frequencies == None:
        # use uniform background
        f = 1.0 / nalphabet
        background_frequencies = [f] * nalphabet
    
    assert len(background_frequencies) == nalphabet
    background_freqs = numpy.array( background_frequencies )
    logodds_matrix = numpy.ones( (motif_width, nalphabet), dtype = numpy.float )

    for x in range( motif_width ):
        # divide by backgound_freqs 
        # divide by column total
        # take log
        logodds_matrix[x] =  numpy.log( counts[x] / counts[x].sum() / background_freqs )
        
    return logodds_matrix
    
def writeMast( outfile, logodds_matrix, alphabet ):
    '''output logodds matrix in MAST format.'''

    
    motif_width, nalphabet, = logodds_matrix.shape
    assert len(alphabet) == nalphabet

    outfile.write( "ALPHABET= %s\n" % alphabet )
    outfile.write( "log-odds matrix: alength= %i w= %i\n" % (nalphabet, motif_width) )
    for row in logodds_matrix:
        outfile.write( " ".join( 
                ["%5.3f" % x for x in row ] ) + "\n")

    outfile.write("\n")

def writeTomTom( outfile, counts_matrix, header = False ):
    '''output counts matrix in tomtom format.

    output counts with columns as motif positions
    and rows as alphabet.
    '''
    
    if header:
        outfile.write( '--------------------------------------------------------------------------------\n')
        outfile.write( '        Motif 1 position-specific probability matrix\n')
        outfile.write( '--------------------------------------------------------------------------------\n')
        outfile.write("letter-probability matrix: alength= %i w= %i nsites= 18 E= 1.0\n" % counts_matrix.shape)

    for row in numpy.transpose( counts_matrix ):
        outfile.write( " ".join( 
                ["%6i" % x for x in row ] ) + "\n")

    outfile.write("\n")

def sequences2motif( outfile, sequences, background_frequencies = None, format = "MAST" ):
    '''write a motif defined by a collection of sequences to outfile.'''

    # collect all letters
    counts = collections.defaultdict( int )
    nsequences = len(sequences)
    seqs = [ re.sub("\s","", x.upper()) for x in sequences ]
    for s in seqs:
        for c in s: counts[c] += 1

    lengths = [len(x) for x in seqs]
    motif_width =min(lengths)
    assert max(lengths) == motif_width, "seqs have to have the same length"
    alphabet = "".join( sorted(counts.keys()) )
    lalphabet = len(alphabet)
    char2index = dict( [(y,x) for x,y in enumerate(alphabet) ] )

    if background_frequencies == None:
        # use uniform background
        f = 1.0 / lalphabet
        background_frequencies = [f] * lalphabet

    if format == "MAST":
        outfile.write( "ALPHABET= %s\n" % alphabet )
        outfile.write( "log-odds matrix: alength= %i w= %i\n" % (lalphabet, motif_width) )
        for r in range( 0, motif_width):
            # use a pseudocount of 1
            row_counts = [ 1 ] * lalphabet
            for s in seqs: row_counts[char2index[s[r]]] += 1
            outfile.write( " ".join(
                ["%5.3f" % (math.log( float(row_counts[x]) / nsequences / background_frequencies[x]) )
                 for x in range(lalphabet)] ) + "\n" )
    elif format == "probability":
        for r in range( 0, motif_width):
            # use a pseudocount of 1
            row_counts = [ 1 ] * lalphabet
            for s in seqs: row_counts[char2index[s[r]]] += 1
            outfile.write( " ".join(
                ["%5.3f" % (float(row_counts[x]) / nsequences ) 
                 for x in range(lalphabet) ] ) + "\n" )
                   
    elif format == "TOMTOM":
        counts = numpy.zeros( (motif_width, lalphabet) )
        for x in range( 0, motif_width):
            for s in seqs:
                counts[x,char2index[s[x]]] += 1

        for y in range(lalphabet):
            for x in range( motif_width):
                outfile.write(" %6i" % counts[x,y])
            outfile.write("\n")





################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
Motifs.py - 
======================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Code
----

'''
import re, collections, math

import Experiment as E
import Genomics, FastaIterator
import numpy

def countMotifs( infile, motifs ):
    '''find regular expression *motifs* in
    sequences within fasta formatted *infile*.
    '''
    
    it = FastaIterator.FastaIterator( infile )
    positions = []
    while 1:
        try:
            seq = it.next()
        except StopIteration:
            break
        if not seq: break
        
        rseq = Genomics.complement( seq.sequence )
        lsequence = len(seq.sequence)
        pos = []
        for motif, pattern in motifs:

            for x in pattern.finditer( seq.sequence ):
                pos.append( ( motif, "+", x.start(), x.end()) )
            for x in pattern.finditer( rseq ):
                pos.append( ( motif, "-", lsequence - x.end(), lsequence - x.start()) )

        positions.append( (seq.title, pos) )

    return positions

def getCounts( matches ):
    '''count numbers of motifs.'''
    counts = collections.defaultdict( int )
    for seq, pos in matches:
        for motif, strand, start, end in pos:
            counts[motif] += 1
    return counts

def getOccurances( matches ):
    '''count numbers of motifs, but only once per sequence'''
    counts = collections.defaultdict( int )
    for seq, pos in matches:
        motifs = set( [x[0] for x in pos ] )
        for m in motifs:
            counts[m] += 1
    return counts

iupacdict = {
    'A':'A',
    'C':'C',
    'G':'G',
    'T':'T',
    'M':'AC',
    'R':'AG',
    'W':'AT',
    'S':'CG',
    'Y':'CT',
    'K':'GT',
    'V':'ACG',
    'H':'ACT',
    'D':'AGT',
    'B':'CGT',
    'X':'ACGT',
    'N':'ACGT'}

regexdict = dict( ( (x[1],x[0]) for x in iupacdict.iteritems() ) )

def iupac2regex( pattern ):
    '''convert iupac to regex pattern'''
    pass

def regex2iupac( pattern ):
    '''convert regex to iupac pattern'''
    
    def _split(p):
        g = p.split("[")
        for c in g[0]: yield c
        for x in g[1:]:
            a,b = x.split("]")
            yield "".join(sorted(a))
            for c in b: yield c
        
    a = []
    for x in _split(pattern):
        a.append( regexdict[x] )
    return "".join( a )
                   




################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
combine_gff.py - merge overlapping intervals
============================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

combine gff features as overlapping regions.

Usage
-----

Example::

   python combine_gff.py --help

Type::

   python combine_gff.py --help

for command line help.

Documentation
-------------

Code
----

'''
import sys
import string
import re
import optparse
import CGAT.Experiment as E
import CGAT.PredictionParser as PredictionParser

if __name__ == "__main__":

    parser = E.OptionParser( version = "%prog version: $Id: combine_gff.py 2781 2009-09-10 11:33:14Z andreas $")

    parser.add_option("-f", "--format", dest="format",
                      help="output format.", type="choice", choices=("flat", "full", "first") )

    parser.set_defaults(
        format="flat"
        )

    (options, args) = E.Start( parser )

    last_e = None
    for line in sys.stdin:
        if line[0] == "#" : continue
        if options.format in ("full", "first" ):
            last_e = GTF.Entry()
        else:
            last_e = GTF.Entry()
        last_e.Read(line)
        break
    
    for line in sys.stdin:
        
        if line[0] == "#" : continue

        if options.format in ("full", "first" ):
            e = GTF.Entry()
        else:
            e = GTF.Entry()
        e.Read(line)

        if not GTF.Overlap( last_e, e):
            print str(last_e)
            last_e = e
        else:
            last_e.start = min( last_e.start, e.start )
            last_e.end = max( last_e.end, e.end )
            if options.format == "full":
                last_e.mInfo += " ; " + e.mInfo
            continue
        
    print str(last_e)

    E.Stop()

        



################################################################################
#   Gene prediction pipeline 
#
#   $Id: trees2trees.py 2782 2009-09-10 11:40:29Z andreas $
#
#   Copyright (C) 2006 Andreas Heger 
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
trees2trees.py - manipulate a collection of trees
=================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This script reads a collection of trees from stdin and
outputs them on stdout. The trees can be filtered by
branch length, monophyly, number of taxa and many more
criteria.

Usage
-----

Example::

   python <script_name>.py --help

Type::

   python <script_name>.py --help

for command line help.

Documentation
-------------

Code
----
'''


import os
import sys
import string
import re
import optparse
import math
import time
import random

USAGE="""python %s [OPTIONS]


""" % sys.argv[0]

import CGAT.Experiment as E
import CGAT.IOTools as IOTools
import CGAT.TreeTools as TreeTools

##------------------------------------------------------------
if __name__ == '__main__':

    parser = E.OptionParser( version = "%prog version: $Id: trees2trees.py 2782 2009-09-10 11:40:29Z andreas $", usage = globals()["__doc__"])

    parser.add_option("-c", "--output-filename-map", dest="output_filename_map", type="string",
                      help="filename of map to output."  )

    parser.add_option("-m", "--method", dest="method", type="choice",
                      choices=("filter", "split"),
                      help="method to use: filter removed trees, while split writes them to individual files. DEFAULT=%default"  )

    parser.add_option("-d", "--output-pattern", dest="output_pattern", type="string",
                      help="filename pattern for output multiple alignment files."  )

    parser.add_option("--filter-terminal-max-length", dest="filter_max_length", type="float",
                      help="remove terminal branches with a branch length larger than this."  )

    parser.add_option("--filter-terminal-min-length", dest="filter_min_length", type="float",
                      help="remove any branches with a branch length smaller than this."  )

    parser.add_option("--filter-min-length", dest="filter_min_length", type="float",
                      help="remove terminal branches with a branch length smaller than this."  )

    parser.add_option("--filter-max-length", dest="filter_min_length", type="float",
                      help="remove any branches with a branch length smaller than this."  )

    parser.add_option("--filter-by-trees", dest="filter_by_trees", type="string", action="append",
                      help="mask branches according to trees. Give filenames with mask trees. These trees need to have the same names and structure as the input trees, but can be in any order."  )

    parser.add_option("--filter-by-monophyly", dest="filter_by_monophyly", type="string",
                      help="only retain trees where the given taxa are monphyletic. Supply taxa as a comma-separated list."  )

    parser.add_option("--min-support", dest="min_support", type="float",
                      help="for monophyly filtering, only accept trees with minimum support."  )

    parser.add_option("--filter-ntaxa", dest="filter_ntaxa", type="int", 
                      help="filter by number of taxa."  )

    parser.add_option("--filter-simple-orthologs", dest="filter_simple_orthologs", action="store_true", 
                      help="filter for trees for simple orhtologs. This works by counting the number of taxa."  )

    parser.add_option("--filter", dest="filter", type="choice",
                      choices=("taxa", "trees"),
                      help="filter removes taxa or whole trees." )

    parser.set_defaults(
        output_pattern="%s.tree",
        output_filename_map = None,
        filter_terminal_max_length = None,
        filter_terminal_min_length = None,
        filter_max_length = None,
        filter_min_length = None,
        method ="split",
        filter = "taxa",
        filtered_branch_length = -999,
        filter_by_trees = [],
        filter_by_monophyly = None,
        filter_ntaxa = None,
        filter_simple_orthologs = None,
        min_support = 0.0,
        regex_species = ("^([^|]+)" ),
        )

    (options, args) = E.Start( parser )

    nexus = TreeTools.Newick2Nexus( sys.stdin )
    
    if options.loglevel >= 1:
        options.stdlog.write("# read %i trees from stdin.\n" % len(nexus.trees))

    ninput, noutput, nskipped = 0, 0, 0
    ndiscarded = 0
    ndiscarded_taxa = 0
    ndiscarded_branches = 0

    extract_species = lambda x: re.search( options.regex_species, x).groups()[0]
    
    if options.filter_by_trees:
        nexus_filter = []
        nexus_maps = []
        for filename in options.filter_by_trees:
            nexus_filter.append( TreeTools.Newick2Nexus( open( filename, "r") ) )
            trees = nexus_filter[-1].trees
            if options.loglevel >=1 :
                options.stdlog.write("# read %i trees for filtering from %s\n" % (len(trees), filename))

            nexus_map = {}
            for x in range( len(trees)):
                nexus_map[trees[x].name] = x
            nexus_maps.append( nexus_map )

    if options.filter_by_monophyly:
        monophyly_taxa = options.filter_by_monophyly.split(",")
        if len(monophyly_taxa) == 0:
            raise "please supply at least two taxa for the monophyly test."
            
    if options.output_filename_map:
        outfile_map = open(options.output_filename_map, "a" )
    else:
        outfile_map = None

    for tree in nexus.trees:

        ninput += 1
        id = tree.name
        has_discarded = False

        if options.filter_ntaxa != None:

            ntaxa = len(tree.get_terminals())
            if ntaxa != options.filter_ntaxa:
                if options.loglevel >= 2:
                    options.stdlog.write("# tree %s: removed because number of taxa (%i) different\n" % \
                                         (id, ntaxa ) )
                has_discarded = True
                
        if options.filter_simple_orthologs:
            ntaxa = len(tree.get_terminals())
            nspecies = len(set(map( lambda x: extract_species(tree.node(x).data.taxon), tree.get_terminals() )))
            if nspecies != ntaxa:
                if options.loglevel >= 2:
                    options.stdlog.write("# tree %s: removed because not a simple ortholog cluster: ntaxa!=nspecies (%i!=%i)\n" % \
                                             (id, ntaxa, nspecies ) )

                has_discarded = True

        if options.filter_terminal_max_length != None:
            for x in tree.get_terminals():
                node = tree.node(x)
                if node.data.branchlength >= options.filter_terminal_max_length:
                    has_discarded = True
                    ndiscarded_taxa += 1                    
                    tree.prune( node.data.taxon )
                    if options.loglevel >= 2:
                        options.stdlog.write("# tree %s: removed taxon %s because terminal branchlength to large: %s\n" % \
                                             (id, node.data.taxon, str(node.data.branchlength)) )

        if options.filter_terminal_min_length != None:
            for x in tree.get_terminals():
                node = tree.node(x)
                if node.data.branchlength <= options.filter_terminal_min_length:
                    has_discarded = True
                    ndiscarded_taxa += 1                    
                    tree.prune( node.data.taxon )
                    if options.loglevel >= 2:
                        options.stdlog.write("# tree %s: removed taxon %s because terminal branchlength to small: %s\n" % \
                                             (id, node.data.taxon, str(node.data.branchlength)) )
                    
        if options.filter_max_length != None:
            for x in tree.get_nodes(tree.root):
                if x == tree.root: continue                
                node = tree.node(x)
                if node.data.branchlength >= options.filter_max_length:
                    has_discarded = True
                    ndiscarded_branches += 1                    
                    if options.loglevel >= 2:
                        options.stdlog.write("# tree %s: removed branch %i because branchlength to large: %s\n" % \
                                             (id, x, tree.name, str(node.data.branchlength)) )
                    node.data.branchlength = options.filtered_branch_length
                    
        if options.filter_min_length != None:
            for x in tree.get_nodes(tree.root):
                if x == tree.root: continue
                node = tree.node(x)
                if node.data.branchlength <= options.filter_min_length:
                    has_discarded = True
                    ndiscarded_branches += 1
                    if options.loglevel >= 2:
                        options.stdlog.write("# tree %s: removed branch %i because internal branchlength too small: %s\n" % \
                                             (id, x, str(node.data.branchlength)) )
                    node.data.branchlength = options.filtered_branch_length
                    
        if options.filter_by_trees:
            found = []
            for y in range(len(nexus_maps)):
                if id in nexus_maps[y]:
                    found.append( (y, nexus_filter[y].trees[nexus_maps[y][id]]) )

            if not found:
                ndiscarded += 1
                continue

            for x in tree.get_nodes(tree.root):
                if x == tree.root: continue
                for y, other_tree in found:
                    other_node = other_tree.node( x )
                    if other_node.data.branchlength == options.filtered_branch_length:
                        node = tree.node(x)
                        if options.loglevel >= 2:
                            options.stdlog.write("# tree %s: removed branch %i because internal branchlength masked by tree %i:%s.\n" % \
                                                 (id, x, y, other_tree.name) )
                        
                        node.data.branchlength = options.filtered_branch_length
                        has_discarded = True
                        ndiscarded_branches += 1
                        break

        if options.filter_by_monophyly:

            terminals = set(map( lambda x: tree.node(x).data.taxon, tree.get_terminals()))
            
            for t in monophyly_taxa:
                if t not in terminals:
                    if options.loglevel >= 2:
                        options.stdlog.write( "taxon %s not in tree %s\n" % (t, tree.name))
                    nskipped += 1
            succ = tree.node(tree.root).succ
            ## use minimum support at root, if it is not the same (if trees
            ## are rooted)
            if len(succ) == 2:
                m = min( map( lambda x: tree.node(x).data.support, succ) )
                for x in succ:
                    tree.node(x).data.support = m
                
            if not TreeTools.IsMonophyleticForTaxa( tree, monophyly_taxa, support=options.min_support ):
                ndiscarded += 1
                continue
            
        if has_discarded:
            ndiscarded += 1
            if options.filter=="trees" or options.filter_ntaxa:
                continue

        if options.method == "split":

            output_filename = re.sub( "%s", id, options.output_pattern )

            dirname = os.path.dirname(output_filename)

            if dirname and not os.path.exists( dirname ):
                os.makedirs( dirname )

            if not os.path.exists( output_filename ):
                outfile = open(output_filename, "w" )
                outfile.write( TreeTools.Tree2Newick( tree ) + "\n" )
                noutput += 1
            else:
                if options.loglevel >= 1:
                    options.stdlog.write("# skipping because output for tree %s already exists: %s\n" % (id, output_filename))                        
                nskipped += 1
                continue

        elif options.method == "filter":
            options.stdout.write( ">%s\n%s\n" % (tree.name, TreeTools.Tree2Newick( tree )) )
            noutput += 1
            
        if outfile_map:
            for t in TreeTools.GetTaxa( tree ):
                outfile_map.write( "%s\t%s\n" % (t, id) )

    if outfile_map:
        outfile_map.close()

    if options.loglevel >= 1:
        options.stdlog.write("# ninput=%i, noutput=%i, nskipped=%i, with_discarded=%i, discarded_taxa=%i, discarded_branches=%i.\n" %\
                             (ninput, noutput, nskipped,
                              ndiscarded, ndiscarded_taxa, ndiscarded_branches))
        
    E.Stop()
    



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
BlatTest.py - 
======================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Code
----

'''
class Error(Exception):
    """Base class for exceptions in this module."""

    def __str__(self):
        return str(self._message)

    def _get_message(self): return self._message
    def _set_message(self, message): self._message = message
    message = property(_get_message, _set_message)

class ParsingError(Error):
    """Exception raised for errors while parsing

    Attributes:
        message -- explanation of the error
    """

    def __init__(self, message, line = None):
        if line:
            self.message = message + " at line:" + line
        else:
            self.message = message

class Match:
    """a psl match.

    Block coordinates are on the forward strand for target and on
    the forward/reverse strand for the query depending on the strand.
    
    The fields mQueryFrom/To and mSbjctFrom/To are always on the forward
    strand.
    """
    
    def __init__(self):
        self.mNMatches = 0
        self.mNMismatches = 0
        self.mNRepMatches = 0
        self.mNns = 0
        self.mQueryNGapsCounts = 0
        self.mQueryNGapsBases = 0
        self.mSbjctNGapsCounts = 0
        self.mSbjctNGapsBases = 0
        self.strand = "+"
        self.mQueryId = ""
        self.mQueryLength = 0
        self.mQueryFrom = 0
        self.mQueryTo = 0
        self.mSbjctId = ""
        self.mSbjctLength = 0
        self.mSbjctFrom = 0
        self.mSbjctTo = 0
        self.mNBlocks = 0
        self.mBlockSizes = []
        self.mQueryBlockStarts = []
        self.mSbjctBlockStarts = []

    def convertCoordinates( self ):
        """convert coordinates.

        This rescales the block positions so that they start at 0 and converts
        the query to forward and the sbjct to forward/reverse coordinates.

        About the psl psl format from the manual at
        http://genome.ucsc.edu/google/goldenPath/help/pslSpec.html

        '''
        In general the coordinates in psl files are "zero based half open." The first base in a sequence is numbered zero rather than one. 
        When representing a range the end coordinate is not included in the range. Thus the first 100 bases of a sequence are represented
        as 0-100, and the second 100 bases are represented as 100-200. 

        There is a another little unusual feature in the .psl format. It has to do with how coordinates are handled on the negative strand. 
        In the qStart/qEnd fields the coordinates are where it matches from the point of view of the forward strand (even when the match is 
        on the reverse strand). However on the qStarts[] list, the coordinates are reversed.
        '''

        I want to work in forward coordinates for the query and forward/reverse coordinates
        for the sbjct.
        
        For a negative strand match, the following is done:
           * invert mSbjctFrom and mSbjctTo with mSbjctLength
           * add block sizes to mQueryStarts and mSbjctStarts
           * invert mQueryStarts and mSbjctStarts
           * reverse blocksize, mQueryStarts and mSbjctStarts
        """

        block_sizes = self.mBlockSize
        query_starts = self.mQueryBlockStarts
        sbjct_starts = self.mSbjctBlockStarts

        if self.mSbjctStrand == "-":
            self.mSbjctFrom, self.mSbjctTo = self.mSbjctLength - self.mSbjctTo, self.mSbjctLength - self.mSbjctFrom
            query_starts = [ self.mQueryLength - x - y for x,y in zip( query_starts, block_sizes) ]
            sbjct_starts = [ self.mSbjctLength - x - y for x,y  in zip( sbjct_starts, block_sizes) ]
            query_starts.reverse()
            sbjct_starts.reverse()

        self.mQueryStarts = map( lambda x: x - self.mQueryFrom, query_starts )
        self.mSbjctStarts = map( lambda x: x - self.mSbjctFrom, sbjct_starts )

    def _switchQueryStrandFlag( self ):
        '''switch the strand flag of the query
        '''

        if self.strand == "-":
            self.strand = "+"
        else:
            self.strand = "-"

    def switchTargetStrand( self ):
        """switch the target strand. 

        Use in cases in which a feature has been defined on the negative target strand
        with reverse coordinates. The result will be the same alignment using forward
        coordinates on the target.

        This method will also update the query strand and coordinates.
        """

        block_sizes = self.mBlockSizes
        query_starts = self.mQueryBlockStarts
        sbjct_starts = self.mSbjctBlockStarts

        # invert alignment
        self.mQueryBlockStarts = [ self.mQueryLength - x - y for x,y  in zip( query_starts, block_sizes) ]
        self.mQueryBlockStarts.reverse()
        self.mSbjctBlockStarts = [ self.mSbjctLength - x - y for x,y  in zip( sbjct_starts, block_sizes) ]
        self.mSbjctBlockStarts.reverse()
        self.mBlockSizes.reverse()

        # update target coordinates only
        self.mSbjctFrom, self.mSbjctTo = self.mSbjctLength - self.mSbjctTo, self.mSbjctLength - self.mSbjctFrom
        self._switchQueryStrandFlag()

    def fromTable( self, data ):
        
        try:
            ( nmatches, nmismatches, nrepmatches, nns,
              query_ngaps_counts, query_ngaps_bases,
              sbjct_ngaps_counts, sbjct_ngaps_bases,
              strand,
              query_id, query_length, query_from, query_to, 
              sbjct_id, sbjct_length, sbjct_from, sbjct_to, 
              nblocks, block_sizes,
              query_block_starts, sbjct_block_starts ) = data[:21]
        except ValueError:
            raise ParsingError( "parsing error: %i fields" % len(data), "\t".join(data ))

        nmatches, nmismatches, nrepmatches, nns = map( \
            int,
            (nmatches, nmismatches, nrepmatches, nns ))

        self.mNMatches = nmatches
        self.mNMismatches = nmismatches
        self.mNRepMatches = nrepmatches
        self.mNns = nns
        self.mQueryNGapsCounts = int(query_ngaps_counts)
        self.mQueryNGapsBases = int(query_ngaps_bases)
        self.mSbjctNGapsCounts = int(sbjct_ngaps_counts)
        self.mSbjctNGapsBases = int(sbjct_ngaps_bases)
        self.strand = strand
        self.mQueryId = query_id
        self.mQueryLength = int(query_length)
        self.mQueryFrom = int(query_from)
        self.mQueryTo = int(query_to)
        self.mSbjctId = sbjct_id
        self.mSbjctLength = int(sbjct_length)
        self.mSbjctFrom = int(sbjct_from)
        self.mSbjctTo = int(sbjct_to)
        self.mNBlocks = int(nblocks)
        self.mBlockSizes = map(int, block_sizes[:-1].split(",") )
        self.mQueryBlockStarts = map(int, query_block_starts[:-1].split(",") )
        self.mSbjctBlockStarts = map(int, sbjct_block_starts[:-1].split(",") )

        ## this makes sure that the block positions are rescaled
        if self.mQueryLength != 0:
            self.mQueryCoverage = 100.0 * (self.mNMismatches + self.mNMatches) / self.mQueryLength
        else:
            self.mQueryCoverage = 0
        
        if self.mSbjctLength != 0:
            self.mSbjctCoverage = 100.0 * (self.mNMismatches + self.mNMatches) / self.mSbjctLength
        else:
            self.mSbjctCoverage = 0

        if nmatches + nmismatches > 0:
            self.mPid = 100.0 * float(nmatches) / (nmatches + nmismatches)
        else:
            self.mPid = 100.0

    def fromMaq( self, maq ):
        """build BLAT entry from a MAQ match.

        see Maq.py
        """
        start = maq.start - 1
        l = maq.mLength
        self.mQueryId = maq.mName
        self.mSbjctId = maq.contig
        self.mQueryFrom = 0
        self.mQueryTo = l
        self.mQueryLength = l
        self.strand = maq.strand
        self.mSbjctFrom = start
        self.mSbjctTo = start + l
        self.mNMismatches = maq.mNMismatches
        self.mNMatches = l - self.mNMismatches
        self.mBlockSizes = [l]
        self.mQueryBlockStarts = [0]
        self.mSbjctBlockStarts = [start]
        self.mNBlocks = 1

    def getBlocks( self ):
        """return a list of aligned blocks."""
        return zip( self.mQueryBlockStarts, self.mSbjctBlockStarts, self.mBlockSizes)

    def __str__(self):
        return "\t".join( map(str, (
                    self.mNMatches,
                    self.mNMismatches,
                    self.mNRepMatches,
                    self.mNns,
                    self.mQueryNGapsCounts,
                    self.mQueryNGapsBases,
                    self.mSbjctNGapsCounts,
                    self.mSbjctNGapsBases,
                    self.strand,
                    self.mQueryId,
                    self.mQueryLength,
                    self.mQueryFrom,
                    self.mQueryTo,
                    self.mSbjctId,
                    self.mSbjctLength,
                    self.mSbjctFrom,
                    self.mSbjctTo,
                    self.mNBlocks,
                    ",".join( map(str,self.mBlockSizes) ) + ",",
                    ",".join( map(str,self.mQueryBlockStarts)) + ",",
                    ",".join( map(str,self.mSbjctBlockStarts)) + "," )))

    def getMapQuery2Target(self):
        """return a map between query to target.

        If the strand is "-", the coordinates for query are on
        the negative strand.
        """

        map_query2target = alignlib.makeAlignmentBlocks()

        f = alignlib.AlignmentFormatBlat( "%i\t%i\t%i\t%i\t%s\t%s\t%s\n" % (\
                min( self.mQueryBlockStarts ),
                max( self.mQueryBlockStarts ),
                min( self.mSbjctBlockStarts ),
                max( self.mSbjctBlockStarts ),
                ",".join( [ str(x) for x in self.mQueryBlockStarts] ) + ",",
                ",".join( [ str(x) for x in self.mSbjctBlockStarts ] ) + ",",
                ",".join( [ str(x) for x in self.mBlockSizes ] ) + "," ))
        f.copy( map_query2target )
        return map_query2target

    def getMapTarget2Query(self):
        """return a map between target to query.

        If the strand is "-", the coordinates for query are on
        the negative strand.
        """

        map_target2query = alignlib.makeAlignmentBlocks()

        f = alignlib.AlignmentFormatBlat( "%i\t%i\t%i\t%i\t%s\t%s\t%s\n" % (\
                min( self.mSbjctBlockStarts ),
                max( self.mSbjctBlockStarts ),
                min( self.mQueryBlockStarts ),
                max( self.mQueryBlockStarts ),
                ",".join( [ str(x) for x in self.mSbjctBlockStarts ] ) + ",",
                ",".join( [ str(x) for x in self.mQueryBlockStarts] ) + ",",
                ",".join( [ str(x) for x in self.mBlockSizes ] ) + "," ))
        f.copy( map_target2query )
        return map_target2query

    def copy( self ):
        return copy.copy(self)

    def fromMap(self, map_query2target, use_strand = None ):
        """return a map between query to target."""

        self.mNMatches = map_query2target.getNumAligned()
        f = alignlib.AlignmentFormatBlat( map_query2target )

        self.mQueryFrom, self.mQueryTo, self.mSbjctFrom, self.mSbjctTo, \
            self.mQueryBlockStarts, self.mSbjctBlockStarts, self.mBlockSizes = str(f).split("\t")
        
        if self.mBlockSizes:
            self.mBlockSizes = map(int, self.mBlockSizes[:-1].split(",") )
            self.mQueryBlockStarts = map(int, self.mQueryBlockStarts[:-1].split(",") )
            self.mSbjctBlockStarts = map(int, self.mSbjctBlockStarts[:-1].split(",") )
        else:
            self.mBlockSizes = []
            self.mQueryBlockStarts = []
            self.mSbjctBlockStarts = []

        self.mNBlocks = len(self.mBlockSizes)

        self.mQueryFrom, self.mQueryTo, self.mSbjctFrom, self.mSbjctTo = \
            map(int,
                (self.mQueryFrom, self.mQueryTo, self.mSbjctFrom, self.mSbjctTo))

        # queryfrom and queryto are always forward strand coordinates
        if use_strand and self.strand == "-":
            self.mQueryFrom, self.mQueryTo = self.mQueryLength - self.mQueryTo, self.mQueryLength - self.mQueryFrom

    def fromPair( self, 
                  query_start, query_size, query_strand, query_seq,
                  target_start, target_size, target_strand, target_seq ):
        '''fill from two aligned sequences.

        Note that sequences are case-sensitive.'''
        
        self.mQueryLength = query_size
        self.mSbjctLength = target_size

        map_query2target = alignlib.makeAlignmentBlocks()
        
        assert len(query_seq) == len(target_seq)

        x,y = query_start, target_start
        nmatches, nmismatches = 0, 0
        for q,t in zip( query_seq, target_seq) :
            tq, tt = q != "-", t != "-"
            if tq and tt: 
                map_query2target.addPair( x,y )
                if q == t: nmatches += 1
                else: nmismatches += 1

            if tq: x += 1
            if tt: y += 1

        self.mNMatches, self.mNMismatches = nmatches, nmismatches
        self.strand = query_strand
        # the following call will set query_from, query_to for the forward strand
        # though block coordinates might be on the negative strand
        self.fromMap( map_query2target, use_strand = True )

        # if target is on negative strand, swop strands
        if target_strand == "-": 
            # swap target strand - this will also swap the query strand
            self.switchTargetStrand()

    def iterator_introns( self ):
        
        b = self.getBlocks()
        last_tend = b[0][1] + b[0][2]
        for qstart, tstart, size in b[1:]:
            yield (last_tend, tstart)
            last_tend = tstart + size
        raise StopIteration

    def iterator_exons( self ):
        
        for qstart, tstart, size in self.getBlocks():
            yield tstart, tstart + size
        raise StopIteration

    def iterator_introns( self ):
        
        b = self.getBlocks()
        last_tend = b[0][1] + b[0][2]
        for qstart, tstart, size in b[1:]:
            yield (last_tend, tstart)
            last_tend = tstart + size
        raise StopIteration

    def iterator_query_exons( self ):
        
        for qstart, tstart, size in self.getBlocks():
            yield qstart, qstart + size
        raise StopIteration

    def iterator_sbjct_exons( self ):
        
        for qstart, tstart, size in self.getBlocks():
            yield tstart, tstart + size
        raise StopIteration

    def iterator_query_introns( self ):
        
        b = self.getBlocks()
        last_qend = b[0][0] + b[0][2]
        for qstart, tstart, size in b[1:]:
            yield (last_qend, qstart)
            last_qend = qstart + size
        raise StopIteration

    def getHeader( self ):

        return \
"""psLayout version 3

match   mis-    rep.    N's     Q gap   Q gap   T gap   T gap   strand  Q       Q       Q       Q       T       T       T       T       block   blockSizes      qStarts  tStarts
        match                   count   bases   count   bases           name    size    start   end     name    size    start   end     count
---------------------------------------------------------------------------------------------------------------------------------------------------------------"""


    def getHeaders(self):
        return ("match", "mismatch", "repeats", "Ns", 
                "qGapCount", "qGapBases", "tGapCount", "tGapBases", "strand",
                "qName", "qSize", "qStart", "qEnd",
                "tName", "tSize", "tStart", "tEnd",
                "blockCount", "blockSizes",
                "qStarts", "tStarts" )                

class MatchPSLX( Match ):

    def __init__(self):
        Match.__init__(self)
        self.mQuerySequence = []
        self.mSbjctSequence = []
        
    def __str__(self):
        return "\t".join( (Match.__str__(self),
                           ",".join( self.mQuerySequence ) + ",",
                           ",".join( self.mSbjctSequence ) + "," ) )

    def fromTable( self, data ):
        
        Match.fromTable( self, data )

        try:
            query_sequence, sbjct_sequence = data[21:23]

        except ValueError:
            raise ParsingError( "parsing error", "\t".join(data ))
        
        self.mQuerySequence = query_sequence[:-1].split(",")
        self.mSbjctSequence = sbjct_sequence[:-1].split(",")


    def getHeaders(self):
        return Match.getHeaders(self) + ("qSequence", "tSequence" )


    def fromPSL( self, other, query_sequence, sbjct_sequence ):
        """fill entry from a psl match.

        sequences are on forward strand starting at
        query_from and sbjct_from, respectively.
        """

        (self.mNMatches,
         self.mNMismatches,
         self.mNRepMatches,
         self.mNns,
         self.mQueryNGapsCounts,
         self.mQueryNGapsBases,
         self.mSbjctNGapsCounts,
         self.mSbjctNGapsBases,
         self.strand,
         self.mQueryId,
         self.mQueryLength,
         self.mQueryFrom,
         self.mQueryTo,
         self.mSbjctId,
         self.mSbjctLength,
         self.mSbjctFrom,
         self.mSbjctTo,
         self.mNBlocks,
         self.mBlockSizes, 
         self.mQueryBlockStarts,
         self.mSbjctBlockStarts ) = (other.mNMatches,
                                     other.mNMismatches,
                                     other.mNRepMatches,
                                     other.mNns,
                                     other.mQueryNGapsCounts,
                                     other.mQueryNGapsBases,
                                     other.mSbjctNGapsCounts,
                                     other.mSbjctNGapsBases,
                                     other.strand,
                                     other.mQueryId,
                                     other.mQueryLength,
                                     other.mQueryFrom,
                                     other.mQueryTo,
                                     other.mSbjctId,
                                     other.mSbjctLength,
                                     other.mSbjctFrom,
                                     other.mSbjctTo,
                                     other.mNBlocks,
                                     other.mBlockSizes, 
                                     other.mQueryBlockStarts,
                                     other.mSbjctBlockStarts )
        
        if self.strand == "-":
            query_sequence = string.translate( query_sequence, string.maketrans("ACGTacgt", "TGCAtgca") )[::-1]

        self.mQuerySequence = []
        self.mSbjctSequence = []
        offset_query = min( self.mQueryBlockStarts)
        offset_sbjct = min( self.mSbjctBlockStarts)

        for start_query, start_sbjct, size in other.getBlocks():
            self.mQuerySequence.append( query_sequence[start_query-offset_query:start_query-offset_query + size ] )
            self.mSbjctSequence.append( sbjct_sequence[start_sbjct-offset_sbjct:start_sbjct-offset_sbjct + size ] )


def _iterate( infile ):
    """iterator over psl output.

    The header is optional.
    """

    format = "ps3"

    while 1:
        line = infile.readline()
        if not line: raise StopIteration

        if line[0] == "#": continue

        if line.startswith("match"):
            continue

        if line.startswith("psLayout version 3"):
            format = "ps3"
            for x in range(4):
                infile.readline()
            continue

        match = Match()
        match.fromTable( line[:-1].split("\t") )
        yield match

class BlatIterator:

    def __init__(self, f, *args, **kwargs):
        self.mIterator = _iterate(f)
        
    def __iter__(self):
        return self

    def next(self):
        try:
            return self.mIterator.next()
        except StopIteration:
            return None

def iterator( infile ):
    """iterate over the contents of a psl file.
    """
    while 1:
        line = infile.readline()
        if not line: raise StopIteration
        if line[0] == "#": continue
        if line.startswith("match"): continue
        if line.startswith("psLayout version 3"):
            for x in range(4): infile.readline()
            continue
        match = Match()
        match.fromTable( line[:-1].split() )
        yield match
        
def iterator_pslx( infile ):
    """iterate over the contents of a pslx file.
    """
    while 1:
        line = infile.readline()
        if not line: raise StopIteration
        if line[0] == "#": continue
        if line.startswith("match"): continue
        if line.startswith("psLayout version 3"):
            for x in range(4): infile.readline()
            continue
        match = MatchPSLX()
        match.fromTable( line[:-1].split() )
        yield match

def iterator_target_overlap( infile, merge_distance ):
    '''iterate over psl formatted infile and return
    blocks of target overlapping alignments.'''

    last_sbjct_id = None
    start = None
    end = None
    matches = []
    processed_contigs = set()

    while 1:

        match = infile.next()
        
        if match == None: break
        
        if match.mSbjctId != last_sbjct_id or match.mSbjctFrom >= (end + merge_distance):
            if last_sbjct_id: yield matches 
            matches = []
            if last_sbjct_id != match.mSbjctId and match.mSbjctId in processed_contigs:
                raise ValueError("input not sorted by target (contig,start): already encountered %s\n%s" %\
                                     (match.mSbjctId, str(match)))

            processed_contigs.add( last_sbjct_id )

            last_sbjct_id = match.mSbjctId
            start = match.mSbjctFrom
            end = match.mSbjctTo
            
        if match.mSbjctFrom < start:
            raise ValueError("input not sorted by target (contig,start): %i < %i\n%s" %\
                                 (match.mSbjctFrom, start, str(match)))

        end = max( match.mSbjctTo, end )
        matches.append(match)
        
    if last_sbjct_id: yield matches

def iterator_query_overlap( infile, merge_distance ):
    '''iterate over psl formatted infile and return
    blocks of target overlapping alignments.'''

    last_query_id = None
    start, end = None, None
    matches = []
    processed_contigs = set()

    while 1:

        match = infile.next()
        
        if match == None: break
        
        if match.mQueryId != last_query_id or match.mQueryFrom >= (end + merge_distance):
            if last_query_id: yield matches 
            matches = []

            if last_query_id != match.mQueryId and match.mQueryId in processed_contigs:
                raise ValueError("input not sorted by query (contig,start): already encountered %s\n%s" %\
                                     (match.mQueryId, str(match)))

            processed_contigs.add( last_query_id )

            last_query_id = match.mQueryId
            start, end = match.mQueryFrom, match.mQueryTo
            
        if match.mQueryFrom < start:
            raise ValueError("input not sorted by query (contig,start): %i < %i\n%s" %\
                                 (match.mQueryFrom, start, str(match)))

        end = max( match.mQueryTo, end )
        matches.append(match)
        
    if last_query_id: yield matches

def iterator_test( infile, report_step = 100000 ):
    '''only output parseable lines from infile.'''
    
    ninput, noutput, nerrors = 0, 0, 0

    while 1:
        try:
            x = infile.next()
        except ParsingError, msg:
            nerrors += 1
            ninput += 1
            E.warn( str(msg) )
            continue
        except StopIteration:
            break

        if not x: break

        ninput += 1        

        if ninput % report_step == 0:
            E.info("progress: ninput=%i, noutput=%i" % (ninput, noutput))

        yield x
        noutput += 1

    E.info( "iterator_test: ninput=%i, noutput=%i, nerrors=%i" % (ninput, noutput,nerrors) )
    
def iterator_per_query( iterator_psl ):
    """iterate over the contents of a psl file per query
    """
    
    data = collections.defaultdict( list )

    for match in iterator_psl:
        data[match.mQueryId].append( match )

    for x in data.values():
        yield x

    raise StopIteration
        
    




################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
csv2csv.py - operate on tables
==============================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

operate on tables.

Usage
-----

Example::

   python csv2csv.py --help

Type::

   python csv2csv.py --help

for command line help.

Documentation
-------------

Code
----

'''
import os
import sys
import string
import re
import getopt
import time
import optparse
import math
import tempfile


"""do some table magic.

--sort: sort table 


"""
import CGAT.Experiment as E
import csv

parser = E.OptionParser( version = "%prog version: $Id: csv2csv.py 2782 2009-09-10 11:40:29Z andreas $")

def ConvertDictionary( d ):
    """tries to convert values in a dictionary.
    """

    rx_int = re.compile("^[+-]*[0-9]+$")
    rx_float = re.compile("^[+-]*[0-9.+-eE]+$")
    for k,v in d.items():
        if rx_int.match( v ):
            d[k] = int(v)
        elif rx_float.match( v ):
            d[k] = float(v)

    return d
    
if __name__ == "__main__":

    parser.add_option( "-s", "--sort", dest="sort", type="string" ,
                       help="fields to take (in sorted order).")

    (options, args) = E.Start( parser, add_csv_options  = True)

    reader = csv.DictReader( sys.stdin, dialect=options.csv_dialect )

    if options.sort:
        fields = options.sort.split(",")
    else:
        fields = None
        
    writer = csv.DictWriter( sys.stdout,
                             fields,
                             dialect=options.csv_dialect,
                             lineterminator = options.csv_lineterminator,
                             extrasaction = 'ignore' )

    print "\t".join(fields)
    
    for row in reader:
        row = ConvertDictionary( row )
        writer.writerow(row)
        
    E.Stop()



#!/usr/bin/env python
################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
filter_reads.py -
=============================================

:Author: ???
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

Usage
-----

Example::

   python cgat_script_template.py --help

Type::

   python cgat_script_template.py --help

for command line help.

Documentation
-------------

Code
----

'''

import os
import sys
import re
import optparse
import sys
import re
from Bio.Seq import Seq
from Bio.Alphabet import generic_dna

import CGAT.Experiment as E

def readread(s):
    return [s.readline(),s.readline(),s.readline(),s.readline()]
 
def writeread(sread,s):
    for i in range(4):
        s.write(sread[i])
 
def filter_pattern(query,s,o):
    sread=readread(s)
    while (sread[0]):
        if (query.search(sread[1])):
            writeread(sread,o)
        sread=readread(s)
 
def filter_low_complexity(s,o):
    sread=readread(s)
    biased = 0
    low_complexity = 0
    total_reads = 0
    remaining_reads = 0
    while (sread[0]):
      total_reads += 1
      my_read = Seq(sread[1], generic_dna)
      a = my_read.count("A")
      c = my_read.count("C")
      t = my_read.count("T")
      g = my_read.count("G")
      seq_len = len(my_read)
      count_list = [a,c,t,g]
      if (count_list.count(0) < 2):
        if (max(a, c, t, g)/seq_len < 0.9):
          writeread(sread,o)
          remaining_reads += 1
        else: biased += 1
      else: low_complexity += 1
      sread=readread(s)
    removed = biased + low_complexity
    sys.stderr.write("Total reads processed: %s\\n" % total_reads)
    sys.stderr.write(r"Low complexity reads removed: %s\n" % low_complexity)
    sys.stderr.write(r"Biased reads removed: %s\n" % biased)
    sys.stderr.write(r"Total reads removed: %s\n" % removed)
    sys.stderr.write(r"Total reads remaining: %s\n" % remaining_reads)

def main( argv = None ):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if not argv: argv = sys.argv

    # setup command line parser
    parser = E.OptionParser( version = "%prog version: $Id$", 
                                    usage = globals()["__doc__"] )

    ## add common options (-h/--help, ...) and parse command line 
    (options, args) = E.Start( parser, argv = argv )

    filter_low_complexity(sys.stdin,sys.stdout)

    ## write footer and output benchmark information.
    E.Stop()

if __name__ == "__main__":
    sys.exit( main( sys.argv) )


	 



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
mask_fasta.py - mask fasta formatted sequences
==============================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

If the file contains multiple sequence entries, it is split
at entry boundaries. Currently, individual entries are not
split.

Usage
-----

Example::

   python mask_fasta.py --help

Type::

   python mask_fasta.py --help

for command line help.

Documentation
-------------

Code
----

'''
import os
import sys
import string
import re
import optparse
import array

import CGAT.Experiment as E

def maskSequence( sequence, regions, mask_char = "N" ):
    """mask sequence with regions."""
    nmasked = 0
    v = array.array( 'c' )
    v.fromstring( sequence)
    errors = []
    for start,end in regions:

        if start < 0 or end > len(sequence):
            errors.append( "out of bounds: %i-%i" % (start, end) )
            continue
        
        for x in range(start,end):
            if v[x] != mask_char:
                v[x] = mask_char
                nmasked += 1
        
    return v.tostring(), nmasked, errors


if __name__ == '__main__':


    parser = E.OptionParser( version = "%prog version: $Id: mask_fasta.py 2782 2009-09-10 11:40:29Z andreas $")

    parser.add_option("-e", "--extend", dest="extend", type="int",
                      help="extend sequences by # residues."  )

    parser.add_option("-o", "--output-pattern", dest="output_pattern", type="string",
                      help="output pattern for the genomic files. Should contain a %i."  )

    parser.add_option("-s", "--sequence", dest="filename_sequence", type="string",
                      help="sequence filename."  )

    parser.add_option("-c", "--chunk-size", dest="chunk_size", type="int",
                      help="chunk size in nucleotides."  )

    parser.add_option("-f", "--format", dest="format", type="choice",
                      choices=("fasta", "ranges"),
                      help="output format options."  )

    parser.add_option("-m", "--mask-regions", dest="filename_mask_regions", type="string",
                      help="mask regions - parameter is a filename of regions with forward strand coordinates in the format 'contig <tab> start <tab> end'."  )

    parser.set_defaults(
        extend = 0,
        format = "fasta",
        filename_sequence = None,
        filename_mask_regions = None,
        mask_char = "N" )

    (options, args) = E.Start( parser, add_pipe_options = True )

    ## read segments to mask
    
    if options.filename_mask_regions:
        mask_regions = {}
        infile = open(options.filename_mask_regions, "r")
        nregions = 0
        for line in infile:
            if line[0] == "#": continue
            contig, start, end = line[:-1].split("\t")
            if contig not in mask_regions: mask_regions[contig] = []
            mask_regions[contig].append( (int(start),int(end) ) )
            nregions += 1

        if options.loglevel >= 1:
            options.stdlog.write("# masking %i regions on %i contigs\n" % (nregions, len(mask_regions)))
            
        for contig, regions in mask_regions.items():
            regions.sort()
            
        infile.close()
    else:
        mask_regions = None
        
    if options.filename_sequence:
        infile = open(options.filename_sequence, "r")
    else:
        infile = sys.stdin

    ## For large genomic sequences, reading the whole data is
    ## a stretch for memory resources. Thus proceed sequence by
    ## sequence

    # read complete genomic sequences
    fragments = []
    sequences = []
    key = None
    description_rest = None
    total_keys, total_written, total_masked, nerrors = 0, 0, 0, 0
    outfile = options.stdout
    
    for line in infile:
        if line[0] == "#": continue
        if line[0] == ">":
            if fragments:

                if options.loglevel >= 1:
                    options.stdlog.write( "# processing sequence %s\n" % key )
                    options.stdlog.flush()
                    
                if key in mask_regions:
                    sequence, nmasked, errors = maskSequence( string.join(fragments, ""), mask_regions[key], options.mask_char)
                    if errors:
                        nerrors += len(errors)
                        options.stdlog.write("# %i errors while masking\n" % len(errors))
                        for e in errors:
                            options.stdlog.write("# %s\n" % e )
                else:
                    sequence = string.join(fragments, "")
                    
                if description_rest:
                    outfile.write(">%s %s\n%s\n" % (key, description_rest, sequence ) )
                else:
                    outfile.write(">%s\n%s\n" % (key, sequence ) )                    
                total_masked += nmasked
                total_keys += 1
                total_written += len(sequence)
                
            fragments = []
            
            x = re.split( "\s", line[1:-1])
            key = x[0]
            description_rest = string.join(x[1:], " ")
            continue
        fragments.append( line[:-1] )

    if fragments:
        if key in mask_regions:
            sequence, nmasked, errors = maskSequence( string.join(fragments, ""), mask_regions[key], options.mask_char)
            if errors:
                nerrors += len(errors)                
                options.stdlog.write("# %i errors while masking\n" % len(errors))
                for e in errors:
                    options.stdlog.write("# %s\n" % e )
        else:
            sequence = string.join(fragments, "")

    total_masked += nmasked
    total_keys += 1
    total_written += len(sequence)

    if description_rest:
        outfile.write(">%s %s\n%s\n" % (key, description_rest, sequence ) )
    else:
        outfile.write(">%s\n%s\n" % (key, sequence ) )                    
    
    if options.filename_sequence: infile.close()

    if options.loglevel >= 1:
        options.stdlog.write( "# nkeys=%i, nwritten=%i, nmasked=%i, nerrors=%i\n" % (total_keys, total_written, total_masked, nerrors) )

    E.Stop()



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
r_table2scatter.py - R based plots and stats
============================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This script reads a table from a file or stdin. 
It can compute various stats (correlations, ...)
and/or plot the data using R.

Usage
-----

Example::

   python r_table2scatter.py --help

Type::

   python r_table2scatter.py --help

for command line help.

Documentation
-------------

Code
----

'''
import os
import sys
import string
import re
import tempfile
import subprocess
import optparse
import time
import math
import code
import CGAT.Experiment as E
import CGAT.MatlabTools as MatlabTools
import numpy
import CGAT.Stats as Stats

from rpy2.robjects import r as R
import rpy2.robjects as ro
import rpy2.robjects.numpy2ri

def readTable( lines,
               assign,
               separator = "\t",
               numeric_type = numpy.float,
               take_columns = "all",
               headers = True,
               row_names = None,
               colours = None,
               ):
    """read a matrix. There probably is a routine for this in Numpy, which
    I haven't found yet. 

    The advantage of using rpy's mechanism is to allow for 
    missing values.

    row_names: column with row names
    """

    # now import R as stdin has been read.
    take = take_columns
    
    handle, name = tempfile.mkstemp()
    os.close(handle)

    if take_columns == "all":
        num_cols = len(string.split(lines[0][:-1], "\t"))
        take = range( 0, num_cols)
    elif take_columns == "all-but-first":
        num_cols = len(string.split(lines[0][:-1], "\t"))        
        take = range( 1, num_cols)
        
    outfile = open(name, "w")
    c = []

    ## delete columns with colour information/legend
    to_delete = []
    if row_names != None:
        legend = []
        if take_columns == "all":
            to_delete.append( row_names )
    else:
        legend = None

    if colours != None:
        if take_columns == "all":
            to_delete.append( colours )

    to_delete.sort()
    to_delete.reverse()
    for x in to_delete: del take[x]

    ## get column headers
    if headers:
        headers = lines[0][:-1].split("\t")
        headers = map( lambda x: headers[x], take )        
        del lines[0]
    
    for l in lines:
        data = [x.strip() for x in l[:-1].split("\t") ]
        if not data or not [ x for x in data if x != ""] : continue
        outfile.write(string.join( map( lambda x: data[x], take ), "\t") + "\n")
        if row_names != None:
            legend.append( data[row_names] )
            
        if colours != None:
            c.append( data[colours] )
        
    outfile.close()

    # rpy.set_default_mode(rpy.NO_CONVERSION)
    # note that the conversion is not perfect. Some missing values are assigned to "nan", while some
    # are -2147483648. They seem to treated correctly, though, within R, but note that when computing
    # something like sum(), the result in python after conversion might be -2147483648.
    if headers:
        matrix = R("""%s <- read.table( '%s', na.string = c("NA", "na", 'nan', 'NaN'), col.names=c('%s'), sep="\t" )""" % (assign, name, "','".join(headers)) )
    else:
        matrix = R("""%s <- read.table( '%s', na.string = c("NA", "na", 'nan', 'NaN'), col.names=headers, sep="\t" )""" % (assign, name) )

    # rpy.set_default_mode(rpy.BASIC_CONVERSION)
    os.remove( name )

    return matrix, headers, c, legend

def writeMatrix( file,
                 matrix,
                 separator = "\t",
                 headers = [],
                 format = "%f" ):
    """write a matrix to file.

    if headers are given, add them to columns and rows.
    """
    
    if headers:
        file.write("\t" + string.join(headers, "\t") + "\n")

    nrows, ncols = matrix.shape

    for x in range(nrows):
        if headers: file.write(headers[x] + "\t")
        file.write(string.join (map(lambda x: format % x, matrix[x]), "\t") + "\n")

def FuncScatterDiagonal( data ):

    R.points( data )
    R.abline( 0, 1 )

def main( argv = None ):
    
    if argv == None: argv = sys.argv

    parser = E.OptionParser( version = "%prog version: $Id: r_table2scatter.py 2782 2009-09-10 11:40:29Z andreas $")

    parser.add_option("-c", "--columns", dest="columns", type="string",
                      help="columns to take from table. Choices are 'all', 'all-but-first' or a ','-separated list of columns." )
    
    parser.add_option( "--logscale", dest="logscale", type="string",
                      help="log-transform one or both axes [default=%Default]."  )

    parser.add_option("-a", "--hardcopy", dest="hardcopy", type="string",
                      help="write hardcopy to file [default=%default].", 
                      metavar = "FILE" )

    parser.add_option("-f", "--file", dest="input_filename", type="string",
                      help="filename with table data [default=%default].",
                      metavar = "FILE")

    parser.add_option("-2", "--file2", dest="input_filename2", type="string",
                      help="additional data file [default=%default].",
                      metavar = "FILE")

    parser.add_option("-s", "--stats", dest="statistics", type="choice",
                      choices=("correlation", "spearman", "pearson", "count"),
                      help="statistical quantities to compute [default=%default]",
                      action = "append")
    
    parser.add_option("-p", "--plot", dest="plot", type="choice",
                      choices=("scatter", "pairs", "panel", "bar", "bar-stacked", 
                               "bar-besides", "1_vs_x", "matched", "boxplot", "scatter+marginal",
                               "scatter-regression" ),
                      help="plots to plot [default=%default]",
                      action = "append")

    parser.add_option("-t", "--threshold", dest="threshold", type="float",
                      help="min threshold to use for counting method [default=%default].")

    parser.add_option("-o", "--colours", dest="colours", type="int",
                      help="column with colour information [default=%default].")

    parser.add_option("-l", "--labels", dest="labels", type="string",
                      help="column labels for x and y in matched plots [default=%default].")

    parser.add_option("-d", "--add-diagonal", dest="add_diagonal", action="store_true",
                      help="add diagonal to plot [default=%default].")
    
    parser.add_option("-e", "--legend", dest="legend", type="int",
                      help="column with legend [default=%default].")

    parser.add_option("-r", "--options", dest="r_options", type="string",
                      help="R plotting options [default=%default].")
    
    parser.add_option("--format", dest="format", type="choice",
                      choices=("full", "sparse"),
                      help="output format [default=%default]." )

    parser.add_option( "--title", dest="title", type="string",
                       help="""plot title [default=%default].""")

    parser.add_option("", "--xrange", dest="xrange", type="string",
                      help="x viewing range of plot [default=%default]."  )

    parser.add_option("", "--yrange", dest="yrange", type="string",
                      help="y viewing range of plot[default=%default]."  )

    parser.add_option( "--allow-empty", dest="fail_on_empty", action="store_false",
                      help="do not fail on empty input [default=%default].")

    parser.add_option( "--fail-on-empty", dest="fail_on_empty", action="store_true",
                      help="fail on empty input [default=%default].")

    parser.set_defaults( \
        hardcopy = None,
        input_filename = "",
        input_filename2 = None,
        columns = "all",
        logscale = None,
        statistics = [],
        plot=[],
        threshold=0.0,
        labels = "x,y",
        colours= None,
        diagonal = False,
        legend = None,
        title = None,
        xrange = None,
        yrange = None,        
        r_options = "",
        fail_on_empty = True,
        format = "full")

    (options, args) = E.Start( parser )

    if len(args) == 1 and not options.input_filename:
        options.input_filename = args[0]

    if options.columns not in ("all", "all-but-first"):
        options.columns = map( lambda x: int(x)-1, options.columns.split(","))
        
    if options.colours: options.colours -= 1
    if options.legend: options.legend -= 1
    
    table ={}
    headers = []

    ## read data matrix
    if options.input_filename:
        lines = open(options.input_filename, "r").readlines()
    else:
        ## note: this will not work for interactive viewing, but
        ## creating hardcopy plots works.
        lines = sys.stdin.readlines()

    lines = filter( lambda x: x[0] != "#", lines)
    
    if len(lines) == 0:
        if options.fail_on_empty:
            raise IOError ( "no input" )
        E.warn( "empty input" )
        E.Stop()
        return

    matrix, headers, colours, legend = readTable( lines,
                                                  "matrix",
                                                  take_columns = options.columns,
                                                  headers=True,
                                                  colours=options.colours,
                                                  row_names = options.legend )

    if options.input_filename2:
        ## read another matrix (should be of the same format.
        matrix2, headers2, colours2, legend2 = readTable( lines,
                                                 "matrix2",
                                                 take_columns = options.columns,
                                                 headers=True,
                                                 colours=options.colours,
                                                 row_names = options.legend )
    R.assign("headers", headers)

    ndata = R( """length( matrix[,1] )""" )[0]

    if options.loglevel >=1:
        options.stdlog.write("# read matrix: %ix%i\n" % (len(headers),ndata) )

    if colours:
        R.assign("colours", colours)

    for method in options.statistics:

        if method == "correlation":
            cor = R.cor(matrix, use="pairwise.complete.obs" )
            writeMatrix( sys.stdout, cor, headers=headers, format = "%5.2f" )

        elif method == "pearson":
            options.stdout.write( "\t".join( ("var1", 
                                              "var2",
                                              "coeff",
                                              "passed",
                                              "pvalue",
                                              "n",
                                              "method",
                                              "alternative" )) + "\n" )
            for x in range(len(headers)-1):
                for y in range( x+1, len(headers)):
                    try:
                        result = R("""cor.test( matrix[,%i], matrix[,%i] )""" % (x + 1, y + 1))
                    except rpy.RPyException, msg:
                        E.warn( "correlation not computed for columns %i(%s) and %i(%s): %s" % (x, headers[x], y, headers[y], msg) )
                        options.stdout.write( "%s\t%s\t%s\t%s\t%s\t%i\t%s\t%s\n" % \
                                                  (headers[x], headers[y],
                                                   "na",
                                                   "na",
                                                   "na",
                                                   0,
                                                   "na",
                                                   "na" ))

                    else:
                        options.stdout.write( "%s\t%s\t%6.4f\t%s\t%e\t%i\t%s\t%s\n" % \
                                                  (headers[x], headers[y],
                                                   result.rx2('estimate').rx2('cor')[0], 
                                                   Stats.getSignificance( float(result.rx2('p.value')[0]) ),
                                                   result.rx2('p.value')[0],
                                                   result.rx2('parameter').rx2('df')[0],
                                                   result.rx2('method')[0], 
                                                   result.rx2('alternative')[0]) )

        elif method == "spearman":
            options.stdout.write( "\t".join( ("var1", "var2",
                                              "coeff",
                                              "passed",
                                              "pvalue",
                                              "method",
                                              "alternative" )) + "\n" )
            for x in range(len(headers)-1):
                for y in range( x+1, len(headers)):
                    result = R("""cor.test( matrix[,%i], matrix[,%i], method='spearman' )""" % (x + 1, y + 1))
                    options.stdout.write( "%s\t%s\t%6.4f\t%s\t%e\t%i\t%s\t%s\n" % \
                                              (headers[x], headers[y],
                                               result['estimate']['rho'], 
                                               Stats.getSignificance( float(result['p.value']) ),
                                               result['p.value'],
                                               result['parameter']['df'],
                                               result['method'], 
                                               result['alternative']))
                    
        elif method == "count":
            ## number of shared elements > threshold
            m, r, c = MatlabTools.ReadMatrix( open(options.input_filename, "r"),
                                              take = options.columns, 
                                              headers=True)
            mask = numpy.greater(m, options.threshold)
            counts = numpy.dot( numpy.transpose(mask), mask)
            writeMatrix( options.stdout, counts, headers=c, format = "%i" )

    if options.plot:

        # remove columns that are completely empty
        if "pairs" in options.plot:
            colsums = R('''colSums( is.na(matrix ))''')
            take = [ x for x in range(len(colsums)) if colsums[x] != ndata ]
            if take:
                E.warn( "removing empty columns %s before plotting" % str(take))
                matrix = R.subset( matrix, select=[ x+1 for x in take] )
                R.assign("""matrix""", matrix )
                headers = [ headers[x] for x in take ]
                if legend:
                    legend = [ headers[x] for x in take ]
        
        if options.r_options:
            extra_options = ", %s" % options.r_options
        else:
            extra_options = ""

        if options.legend != None and len(legend):
            extra_options += ", legend=c('%s')" % "','".join(legend)
            
        if options.labels:
            xlabel, ylabel = options.labels.split(",")
            extra_options += ", xlab='%s', ylab='%s'" % (xlabel, ylabel)
        else:
            xlabel, ylabel = "", ""

        if options.colours:
            extra_options += ", col=colours"
            
        if options.logscale:
            extra_options += ", log='%s'" % options.logscale

        if options.xrange:
            extra_options += ", xlim=c(%f,%f)" % tuple(map(float, options.xrange.split(",") ) )

        if options.yrange:
            extra_options += ", ylim=c(%f,%f)" % tuple(map(float, options.yrange.split(",") ) )

        if options.hardcopy:
            if options.hardcopy.endswith(".eps"): 
                R.postscript(options.hardcopy)
            elif options.hardcopy.endswith(".png"): 
                R.png(options.hardcopy, width=1024, height=768, type="cairo")
            elif options.hardcopy.endswith(".jpg"): 
                R.jpg(options.hardcopy, width=1024, height=768, type="cairo")

        for method in options.plot:

            if ndata < 100:
                point_size = "1"
                pch = "o"
            elif ndata < 1000:
                point_size = "1"
                pch = "o"
            else:
                point_size = "0.5"
                pch = "."

            if method == "scatter":
                R("""plot( matrix[,1], matrix[,2], cex=%s, pch="o" %s)""" % (point_size, extra_options) )

            if method == "scatter-regression":
                R("""plot( matrix[,1], matrix[,2], cex=%s, pch="o" %s)""" % (point_size, extra_options) )
                dat = R("""dat <- data.frame(x = matrix[,1], y = matrix[,2])""")
                R("""new <- data.frame(x = seq( min(matrix[,1]), max(matrix[,1]), (max(matrix[,1]) - min(matrix[,1])) / 100))""")
                mod = R("""mod <- lm( y ~ x, dat)""")
                R("""predict(mod, new, se.fit = TRUE)""")
                R("""pred.w.plim <- predict(mod, new, interval="prediction")""")
                R("""pred.w.clim <- predict(mod, new, interval="confidence")""")
                R("""matpoints(new$x,cbind(pred.w.clim, pred.w.plim[,-1]), lty=c(1,2,2,3,3), type="l")""")
                R.mtext(
                    "y = %f * x + %f, r=%6.4f, n=%i" % (mod["coefficients"]["x"], 
                                                        mod["coefficients"]["(Intercept)"], 
                                                        R("""cor( dat )[2]"""), 
                                                        ndata ),
                    3,
                    cex = 1.0)

            elif method == "pairs":
                if options.add_diagonal:
                    R( """panel.hist <- function( x,y,...  ) { points(x,y,...); abline(0,1); }""" )
                else:
                    R( """panel.hist <- function( x,y,...  ) { points(x,y,...); }""" )
                    
                # There used to be a argument na_action="na.omit", but removed this
                # as there appeared error messages saying "na.action is not a graphical parameter"
                # and the plots showed occasionally the wrong scale.
                # cex=point_size also caused trouble (error message: "X11 used font size 8 when 2 was requested" or similar)
                if options.colours:
                    R.pairs( matrix, 
                             pch=pch, 
                             col=colours,
                             main=options.title,
                             panel="panel.hist",
                             labels=headers, 
                             cex_labels=2.0 )
                else:
                    R.pairs( matrix, 
                             pch=pch,
                             panel="panel.hist", 
                             main=options.title,
                             labels=headers, 
                             cex_labels=2.0 )

            elif method == "boxplot":
                extra_options += ",main='%s'" % options.title

                # set vertical orientation
                if max( [len(x) for x in headers] ) > 40 / len(headers):
                    # remove xlabel:
                    extra_options = re.sub( ", xlab='[^']+'", "", extra_options )
                    extra_options += ", names.arg=headers, las=2"
                    R("""op <- par(mar=c(11,4,4,2))""") # the 10 allows the names.arg below the barplot

                R("""boxplot( matrix %s)""" % extra_options)
                
            elif method == "bar" or method == "bar-stacked":
                if not options.colours:
                    extra_options += ", col=rainbow(5)"

                # set vertical orientation
                if max( [len(x) for x in headers] ) > 40 / len(headers):
                    # remove xlabel:
                    extra_options = re.sub( ", xlab='[^']+'", "", extra_options )
                    extra_options += ", names.arg=headers, las=2"
                    R("""op <- par(mar=c(11,4,4,2))""") # the 10 allows the names.arg below the barplot
                
                R("""barplot(as.matrix(matrix), %s)""" % extra_options)


            elif method == "bar-besides":
                if not options.colours:
                    extra_options += ", col=rainbow(%i)" % ndata

                # set vertical orientation
                if max( [len(x) for x in headers] ) > 40 / len(headers):
                    # remove xlabel:
                    extra_options = re.sub( ", xlab='[^']+'", "", extra_options )
                    extra_options += ", names.arg=headers, las=2"
                    R("""op <- par(mar=c(11,4,4,2))""") # the 10 allows the names.arg below the barplot

                R("""barplot(as.matrix(matrix), beside=TRUE %s)""" % extra_options)

            elif method == "scatter+marginal":

                if options.title:
                    # set the size of the outer margins - the title needs to be added at the end
                    # after plots have been created
                    R.par(oma=R.c(0,0,4,0) )                     

                R( """matrix""" )
                R( """
x <- matrix[,1];
y <- matrix[,2];
xhist <- hist(x, breaks=20, plot=FALSE);
yhist <- hist(y, breaks=20, plot=FALSE);
top <- max(c(xhist$counts, yhist$counts));
nf <- layout(matrix(c(2,0,1,3),2,2,byrow=TRUE), c(3,1), c(1,3), respect=TRUE );
par(mar=c(3,3,1,1)) ;
plot(x, y, cex=%s, pch="o" %s) ;
par(mar=c(0,3,1,1)) ;
barplot(xhist$counts, axes=FALSE, ylim=c(0, top), space=0 ) ;
par(mar=c(3,0,1,1)) ;
title(main='%s');
barplot(yhist$counts, axes=FALSE, xlim=c(0, top), space=0, horiz=TRUE ) ;
title(main='%s');
""" % (point_size, extra_options, xlabel, ylabel))

                if options.title:
                    R.mtext( options.title,3,outer=True,line=1,cex=1.5)  

            elif method in ("panel", "1_vs_x", "matched") :

                if method == "panel":
                    pairs = []
                    for x in range(len(headers)-1):
                        for y in range( x+1, len(headers)):
                            pairs.append( (x, y) )
                        
                elif method == "1_vs_x":
                    pairs = []
                    for x in range(1, len(headers)):
                        pairs.append( (0, x) )
                    
                # print matching columns
                elif method == "matched":
                    pairs = []
                    for x in range(len(headers)-1):
                        for y in range( x+1, len(headers)):
                            if headers[x] == headers[y]:
                                pairs.append( (x, y) )
                                break

                w = int(math.ceil(math.sqrt(len(pairs))))
                h = int(math.ceil(float(len(pairs)) / w))

                PosInf = 1e300000
                NegInf = -1e300000             

                xlabel, ylabel = options.labels.split(",")

                R( """layout(matrix(seq(1,%i), %i, %i, byrow = TRUE))""" % (w*h, w, h))
                for a,b in pairs:
                    new_matrix = filter( lambda x: \
                                         x[0] not in (float("nan"), PosInf, NegInf) and \
                                         x[1] not in (float("nan"), PosInf, NegInf), \
                                         zip(matrix[a].values()[0], matrix[b].values()[0]) )
                    try:
                        R( """plot(matrix[,%i], matrix[,%i], main='%s versus %s', cex=0.5, pch=".", xlab='%s', ylab='%s' )""" % (\
                            a+1,b+1,headers[b], headers[a], xlabel, ylabel))
                    except rpy.RException, msg:
                        print "could not plot %s versus %s: %s" % (headers[b], headers[a], msg)

        if options.hardcopy:
            R['dev.off']()

    E.info( "matrix added as >matrix< in R.")
        
    if not options.hardcopy:
        if options.input_filename:
            interpreter = code.InteractiveConsole(globals())
            interpreter.interact()
        else:
            E.info( "can not start new interactive session as input has come from stdin.")

    E.Stop()

if __name__ == "__main__":
    main() 
